from typing import Any, Dict, List, Union

import httpx

from ...client import Client
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    product_code: List[str],
    start_time: Union[Unset, None, str] = UNSET,
    end_time: Union[Unset, None, str] = UNSET,
    start_time_local: Union[Unset, None, str] = UNSET,
    end_time_local: Union[Unset, None, str] = UNSET,
    min_availability: Union[Unset, None, int] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    offset: Union[Unset, None, int] = 0,
) -> Dict[str, Any]:
    url = "{}/availability".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    json_product_code = product_code

    params["productCode"] = json_product_code

    params["startTime"] = start_time

    params["endTime"] = end_time

    params["startTimeLocal"] = start_time_local

    params["endTimeLocal"] = end_time_local

    params["minAvailability"] = min_availability

    params["limit"] = limit

    params["offset"] = offset

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _build_response(*, response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=None,
    )


def sync_detailed(
    *,
    client: Client,
    product_code: List[str],
    start_time: Union[Unset, None, str] = UNSET,
    end_time: Union[Unset, None, str] = UNSET,
    start_time_local: Union[Unset, None, str] = UNSET,
    end_time_local: Union[Unset, None, str] = UNSET,
    min_availability: Union[Unset, None, int] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    offset: Union[Unset, None, int] = 0,
) -> Response[Any]:
    """Search availability

     This service returns availability information for a specific date range. The service response
    contains a list of sessions, including their availability and pricing details.<br>
    <p>Pricing in the session can be different than the pricing of the products, in a case of a dynamic
    pricing when a supplier overrides a price for a specific session or a ticket type.</p>
    <p>In case of multiple products sharing this session, a session will contain price overrides for all
    of the shared products. Therefore it is necessary to filer only the price options matching the
    chosen product code on the client side, when displaying available price options to a customer.</p>

    Args:
        product_code (List[str]):
        start_time (Union[Unset, None, str]):
        end_time (Union[Unset, None, str]):
        start_time_local (Union[Unset, None, str]):
        end_time_local (Union[Unset, None, str]):
        min_availability (Union[Unset, None, int]):
        limit (Union[Unset, None, int]):
        offset (Union[Unset, None, int]):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        client=client,
        product_code=product_code,
        start_time=start_time,
        end_time=end_time,
        start_time_local=start_time_local,
        end_time_local=end_time_local,
        min_availability=min_availability,
        limit=limit,
        offset=offset,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


async def asyncio_detailed(
    *,
    client: Client,
    product_code: List[str],
    start_time: Union[Unset, None, str] = UNSET,
    end_time: Union[Unset, None, str] = UNSET,
    start_time_local: Union[Unset, None, str] = UNSET,
    end_time_local: Union[Unset, None, str] = UNSET,
    min_availability: Union[Unset, None, int] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    offset: Union[Unset, None, int] = 0,
) -> Response[Any]:
    """Search availability

     This service returns availability information for a specific date range. The service response
    contains a list of sessions, including their availability and pricing details.<br>
    <p>Pricing in the session can be different than the pricing of the products, in a case of a dynamic
    pricing when a supplier overrides a price for a specific session or a ticket type.</p>
    <p>In case of multiple products sharing this session, a session will contain price overrides for all
    of the shared products. Therefore it is necessary to filer only the price options matching the
    chosen product code on the client side, when displaying available price options to a customer.</p>

    Args:
        product_code (List[str]):
        start_time (Union[Unset, None, str]):
        end_time (Union[Unset, None, str]):
        start_time_local (Union[Unset, None, str]):
        end_time_local (Union[Unset, None, str]):
        min_availability (Union[Unset, None, int]):
        limit (Union[Unset, None, int]):
        offset (Union[Unset, None, int]):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        client=client,
        product_code=product_code,
        start_time=start_time,
        end_time=end_time,
        start_time_local=start_time_local,
        end_time_local=end_time_local,
        min_availability=min_availability,
        limit=limit,
        offset=offset,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.booking_field_field_type import BookingFieldFieldType
from ..types import UNSET, Unset

T = TypeVar("T", bound="BookingField")


@attr.s(auto_attribs=True)
class BookingField:
    """An information about a booking or a participant, based on context - either nested in a booking or a participant
    object (I.e. "First Name", "Dietary requirements", "Hotel Pickup").<p><ul>How to handle the booking fields:
    <li>requiredPerXXX is true: the fields is mandatory for the booking, a booking with missing required field will
    fail</li> 	<li>visiblePerXXX is true: the fields is optional, a customer should be asked to fill it up, but the
    booking will not fail if it's empty or not sent</li>	<li>visiblePerXXX is false AND requiredPerXXX is false: the
    field for this product is not visible, nor required, you can skip it - do not display it to a user</li></ul><ul>How
    to send the booking fields in a booking request:	<li>fields XXXperParticipant are sent for each pax in the
    participants array, as "fields"</li> 	<li>fields XXXperBooking are sent only once, in the root booking object, as
    "fields"</li></ul></p>

        Attributes:
            field_type (Union[Unset, BookingFieldFieldType]): Booking field type which determines its format. See <a
                href="/guides/API%20Related%20Articles/Booking%20Fields%20Format">Booking Fields Format</a>
            label (Union[Unset, str]): Field label that can be shown to customers
            list_options (Union[Unset, str]): If this field only allows limited values to be selected from a list, they'll
                be included in this string, separated by \r\n
            required_per_booking (Union[Unset, bool]): true if this field must be populated once per booking, regardless of
                the number of items or participants. It should be in Booking.fields<p><i>Currently, required fields are not
                validated when a booking is created though public API, however, it's a good practice to support them in your
                client code <b>However, soon the required fields will be enforced for public API booking.</b>.</i></p>
            required_per_participant (Union[Unset, bool]): true if this field must be populated for each participant. It
                should be in Booking.BookingItem.Participant.fields.<p><i>Currently, required fields are not validated when a
                booking is created through public API, however, it's a good practice to support them in your client code.
                <b>However, soon the required fields will be enforced for public API booking.</b></i></p>
            value (Union[Unset, str]): Value entered by the customer for this field
            visible_per_booking (Union[Unset, bool]): true if this field should be asked once per booking, regardless of the
                number of items or participants. It should be in Booking.fields
            visible_per_participant (Union[Unset, bool]): true if this field should be asked for each participant when doing
                a booking. It should be in Booking.BookingItem.Participant.fields.
    """

    field_type: Union[Unset, BookingFieldFieldType] = UNSET
    label: Union[Unset, str] = UNSET
    list_options: Union[Unset, str] = UNSET
    required_per_booking: Union[Unset, bool] = UNSET
    required_per_participant: Union[Unset, bool] = UNSET
    value: Union[Unset, str] = UNSET
    visible_per_booking: Union[Unset, bool] = UNSET
    visible_per_participant: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        field_type: Union[Unset, str] = UNSET
        if not isinstance(self.field_type, Unset):
            field_type = self.field_type.value

        label = self.label
        list_options = self.list_options
        required_per_booking = self.required_per_booking
        required_per_participant = self.required_per_participant
        value = self.value
        visible_per_booking = self.visible_per_booking
        visible_per_participant = self.visible_per_participant

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if field_type is not UNSET:
            field_dict["fieldType"] = field_type
        if label is not UNSET:
            field_dict["label"] = label
        if list_options is not UNSET:
            field_dict["listOptions"] = list_options
        if required_per_booking is not UNSET:
            field_dict["requiredPerBooking"] = required_per_booking
        if required_per_participant is not UNSET:
            field_dict["requiredPerParticipant"] = required_per_participant
        if value is not UNSET:
            field_dict["value"] = value
        if visible_per_booking is not UNSET:
            field_dict["visiblePerBooking"] = visible_per_booking
        if visible_per_participant is not UNSET:
            field_dict["visiblePerParticipant"] = visible_per_participant

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _field_type = d.pop("fieldType", UNSET)
        field_type: Union[Unset, BookingFieldFieldType]
        if isinstance(_field_type, Unset):
            field_type = UNSET
        else:
            field_type = BookingFieldFieldType(_field_type)

        label = d.pop("label", UNSET)

        list_options = d.pop("listOptions", UNSET)

        required_per_booking = d.pop("requiredPerBooking", UNSET)

        required_per_participant = d.pop("requiredPerParticipant", UNSET)

        value = d.pop("value", UNSET)

        visible_per_booking = d.pop("visiblePerBooking", UNSET)

        visible_per_participant = d.pop("visiblePerParticipant", UNSET)

        booking_field = cls(
            field_type=field_type,
            label=label,
            list_options=list_options,
            required_per_booking=required_per_booking,
            required_per_participant=required_per_participant,
            value=value,
            visible_per_booking=visible_per_booking,
            visible_per_participant=visible_per_participant,
        )

        booking_field.additional_properties = d
        return booking_field

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.net_rate import NetRate
from ..models.product_rate_commission_type import ProductRateCommissionType
from ..types import UNSET, Unset

T = TypeVar("T", bound="ProductRate")


@attr.s(auto_attribs=True)
class ProductRate:
    """A ProductRate is used to map a product and its associated value commission

    Attributes:
        commission_type (Union[Unset, ProductRateCommissionType]): Commission type: PERCENTAGE, NET_RATE
        net_rates (Union[Unset, List[NetRate]]): List of Net rates with its associated price option label e.g. Adult
            $20, Child $10 etc. This is mandatory if Commission Type is NET_RATE
        percentage_commission (Union[Unset, float]): Percentage value of the commission. This should be mandatory if
            Commission Type is PERCENTAGE
        percentage_include_extras (Union[Unset, bool]): Includes extras, This is mandatory if Commission Type is
            PERCENTAGE. If true, the product's extras will be included in the agent commission, otherwise the commission
            will be calculated based on the product price only.
        product_code (Union[Unset, str]): Product's product code
    """

    commission_type: Union[Unset, ProductRateCommissionType] = UNSET
    net_rates: Union[Unset, List[NetRate]] = UNSET
    percentage_commission: Union[Unset, float] = UNSET
    percentage_include_extras: Union[Unset, bool] = UNSET
    product_code: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        commission_type: Union[Unset, str] = UNSET
        if not isinstance(self.commission_type, Unset):
            commission_type = self.commission_type.value

        net_rates: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.net_rates, Unset):
            net_rates = []
            for net_rates_item_data in self.net_rates:
                net_rates_item = net_rates_item_data.to_dict()

                net_rates.append(net_rates_item)

        percentage_commission = self.percentage_commission
        percentage_include_extras = self.percentage_include_extras
        product_code = self.product_code

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if commission_type is not UNSET:
            field_dict["commissionType"] = commission_type
        if net_rates is not UNSET:
            field_dict["netRates"] = net_rates
        if percentage_commission is not UNSET:
            field_dict["percentageCommission"] = percentage_commission
        if percentage_include_extras is not UNSET:
            field_dict["percentageIncludeExtras"] = percentage_include_extras
        if product_code is not UNSET:
            field_dict["productCode"] = product_code

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _commission_type = d.pop("commissionType", UNSET)
        commission_type: Union[Unset, ProductRateCommissionType]
        if isinstance(_commission_type, Unset):
            commission_type = UNSET
        else:
            commission_type = ProductRateCommissionType(_commission_type)

        net_rates = []
        _net_rates = d.pop("netRates", UNSET)
        for net_rates_item_data in _net_rates or []:
            net_rates_item = NetRate.from_dict(net_rates_item_data)

            net_rates.append(net_rates_item)

        percentage_commission = d.pop("percentageCommission", UNSET)

        percentage_include_extras = d.pop("percentageIncludeExtras", UNSET)

        product_code = d.pop("productCode", UNSET)

        product_rate = cls(
            commission_type=commission_type,
            net_rates=net_rates,
            percentage_commission=percentage_commission,
            percentage_include_extras=percentage_include_extras,
            product_code=product_code,
        )

        product_rate.additional_properties = d
        return product_rate

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.customer_gender import CustomerGender
from ..models.customer_title import CustomerTitle
from ..types import UNSET, Unset

T = TypeVar("T", bound="Customer")


@attr.s(auto_attribs=True)
class Customer:
    """The customer is the person making the booking, and most of the time paying for it.<br>It differs from Participants,
    who are the people attending a tour

        Attributes:
            about_us (Union[Unset, str]): How did you hear about us?
            address_line (Union[Unset, str]): Address
            address_line_2 (Union[Unset, str]): Extended Address
            city (Union[Unset, str]): City/Town/Suburb
            company_name (Union[Unset, str]): Company name
            country_code (Union[Unset, str]): 2 letter ISO country code
            dob (Union[Unset, datetime.datetime]): Date of birth
            email (Union[Unset, str]): Email
            fax (Union[Unset, str]): Fax number
            first_name (Union[Unset, str]): First name
            gender (Union[Unset, CustomerGender]): Gender: MALE or FEMALE
            id (Union[Unset, int]): Rezdy internal ID of the customer
            last_name (Union[Unset, str]): Last Name
            marketing (Union[Unset, bool]): Agree to receive marketing emails
            middle_name (Union[Unset, str]): Middle name
            mobile (Union[Unset, str]): Mobile phone number
            name (Union[Unset, str]): Full name - generated from first/middle/last names
            newsletter (Union[Unset, bool]): Subscribe to the newsletter
            phone (Union[Unset, str]): Preferred Phone number
            post_code (Union[Unset, str]): Postcode / ZIP
            preferred_language (Union[Unset, str]): Preferred language. Should be a 2 letter ISO country code
            skype (Union[Unset, str]): Skype alias
            state (Union[Unset, str]): State/County/Region
            title (Union[Unset, CustomerTitle]): Title
    """

    about_us: Union[Unset, str] = UNSET
    address_line: Union[Unset, str] = UNSET
    address_line_2: Union[Unset, str] = UNSET
    city: Union[Unset, str] = UNSET
    company_name: Union[Unset, str] = UNSET
    country_code: Union[Unset, str] = UNSET
    dob: Union[Unset, datetime.datetime] = UNSET
    email: Union[Unset, str] = UNSET
    fax: Union[Unset, str] = UNSET
    first_name: Union[Unset, str] = UNSET
    gender: Union[Unset, CustomerGender] = UNSET
    id: Union[Unset, int] = UNSET
    last_name: Union[Unset, str] = UNSET
    marketing: Union[Unset, bool] = UNSET
    middle_name: Union[Unset, str] = UNSET
    mobile: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    newsletter: Union[Unset, bool] = UNSET
    phone: Union[Unset, str] = UNSET
    post_code: Union[Unset, str] = UNSET
    preferred_language: Union[Unset, str] = UNSET
    skype: Union[Unset, str] = UNSET
    state: Union[Unset, str] = UNSET
    title: Union[Unset, CustomerTitle] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        about_us = self.about_us
        address_line = self.address_line
        address_line_2 = self.address_line_2
        city = self.city
        company_name = self.company_name
        country_code = self.country_code
        dob: Union[Unset, str] = UNSET
        if not isinstance(self.dob, Unset):
            dob = self.dob.isoformat()

        email = self.email
        fax = self.fax
        first_name = self.first_name
        gender: Union[Unset, str] = UNSET
        if not isinstance(self.gender, Unset):
            gender = self.gender.value

        id = self.id
        last_name = self.last_name
        marketing = self.marketing
        middle_name = self.middle_name
        mobile = self.mobile
        name = self.name
        newsletter = self.newsletter
        phone = self.phone
        post_code = self.post_code
        preferred_language = self.preferred_language
        skype = self.skype
        state = self.state
        title: Union[Unset, str] = UNSET
        if not isinstance(self.title, Unset):
            title = self.title.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if about_us is not UNSET:
            field_dict["aboutUs"] = about_us
        if address_line is not UNSET:
            field_dict["addressLine"] = address_line
        if address_line_2 is not UNSET:
            field_dict["addressLine2"] = address_line_2
        if city is not UNSET:
            field_dict["city"] = city
        if company_name is not UNSET:
            field_dict["companyName"] = company_name
        if country_code is not UNSET:
            field_dict["countryCode"] = country_code
        if dob is not UNSET:
            field_dict["dob"] = dob
        if email is not UNSET:
            field_dict["email"] = email
        if fax is not UNSET:
            field_dict["fax"] = fax
        if first_name is not UNSET:
            field_dict["firstName"] = first_name
        if gender is not UNSET:
            field_dict["gender"] = gender
        if id is not UNSET:
            field_dict["id"] = id
        if last_name is not UNSET:
            field_dict["lastName"] = last_name
        if marketing is not UNSET:
            field_dict["marketing"] = marketing
        if middle_name is not UNSET:
            field_dict["middleName"] = middle_name
        if mobile is not UNSET:
            field_dict["mobile"] = mobile
        if name is not UNSET:
            field_dict["name"] = name
        if newsletter is not UNSET:
            field_dict["newsletter"] = newsletter
        if phone is not UNSET:
            field_dict["phone"] = phone
        if post_code is not UNSET:
            field_dict["postCode"] = post_code
        if preferred_language is not UNSET:
            field_dict["preferredLanguage"] = preferred_language
        if skype is not UNSET:
            field_dict["skype"] = skype
        if state is not UNSET:
            field_dict["state"] = state
        if title is not UNSET:
            field_dict["title"] = title

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        about_us = d.pop("aboutUs", UNSET)

        address_line = d.pop("addressLine", UNSET)

        address_line_2 = d.pop("addressLine2", UNSET)

        city = d.pop("city", UNSET)

        company_name = d.pop("companyName", UNSET)

        country_code = d.pop("countryCode", UNSET)

        _dob = d.pop("dob", UNSET)
        dob: Union[Unset, datetime.datetime]
        if isinstance(_dob, Unset):
            dob = UNSET
        else:
            dob = isoparse(_dob)

        email = d.pop("email", UNSET)

        fax = d.pop("fax", UNSET)

        first_name = d.pop("firstName", UNSET)

        _gender = d.pop("gender", UNSET)
        gender: Union[Unset, CustomerGender]
        if isinstance(_gender, Unset):
            gender = UNSET
        else:
            gender = CustomerGender(_gender)

        id = d.pop("id", UNSET)

        last_name = d.pop("lastName", UNSET)

        marketing = d.pop("marketing", UNSET)

        middle_name = d.pop("middleName", UNSET)

        mobile = d.pop("mobile", UNSET)

        name = d.pop("name", UNSET)

        newsletter = d.pop("newsletter", UNSET)

        phone = d.pop("phone", UNSET)

        post_code = d.pop("postCode", UNSET)

        preferred_language = d.pop("preferredLanguage", UNSET)

        skype = d.pop("skype", UNSET)

        state = d.pop("state", UNSET)

        _title = d.pop("title", UNSET)
        title: Union[Unset, CustomerTitle]
        if isinstance(_title, Unset):
            title = UNSET
        else:
            title = CustomerTitle(_title)

        customer = cls(
            about_us=about_us,
            address_line=address_line,
            address_line_2=address_line_2,
            city=city,
            company_name=company_name,
            country_code=country_code,
            dob=dob,
            email=email,
            fax=fax,
            first_name=first_name,
            gender=gender,
            id=id,
            last_name=last_name,
            marketing=marketing,
            middle_name=middle_name,
            mobile=mobile,
            name=name,
            newsletter=newsletter,
            phone=phone,
            post_code=post_code,
            preferred_language=preferred_language,
            skype=skype,
            state=state,
            title=title,
        )

        customer.additional_properties = d
        return customer

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.booking_field_create import BookingFieldCreate
from ..models.price_option_create import PriceOptionCreate
from ..models.product_update_request_barcode_output_type import ProductUpdateRequestBarcodeOutputType
from ..models.product_update_request_confirm_mode import ProductUpdateRequestConfirmMode
from ..types import UNSET, Unset

T = TypeVar("T", bound="ProductUpdateRequest")


@attr.s(auto_attribs=True)
class ProductUpdateRequest:
    """Partial product model containing all fields which are currently supported in product create via API.

    Attributes:
        booking_fields (List[BookingFieldCreate]): List of booking fields required for this product.
        confirm_mode (ProductUpdateRequestConfirmMode): Confirmation mode. Determines if bookings are automatically
            confirmed or it they are pending.
        description (str): Long product description, is between 100 and 15000 characters.
        short_description (str): Product description is between 15 and 240 characters.
        advertised_price (Union[Unset, float]): General price indication for this product. It represents a display price
            only, therefore it does not affect a real booking price, which is calculated based on the price options.
        barcode_output_type (Union[Unset, ProductUpdateRequestBarcodeOutputType]): Specifies how to output the barcodes
            when this product is booked. Valid types are:<br><li>PARTICIPANT: Barcodes will be generated by rezdy for each
            participant when an booking is created for this product</li><li>ORDER: Barcodes will be generated by rezdy per
            booking</li>
        confirm_mode_min_participants (Union[Unset, int]): If confirmMode is MANUAL_THEN_AUTO or AUTO_THEN_MANUAL,
            determines the minimum number of participants per booking to trigger the change.
        duration_minutes (Union[Unset, int]): Duration of the product in minutes.
        internal_code (Union[Unset, str]): Supplier-defined product code, used internally by the supplier.
        minimum_notice_minutes (Union[Unset, int]): Minimum book ahead interval for he product in minutes.
        name (Union[Unset, str]): Product name
        pickup_id (Union[Unset, int]): Pickup ID for this product.
        price_options (Union[Unset, List[PriceOptionCreate]]): List of price options for this product.
        terms (Union[Unset, str]): Specific terms and conditions for this product.
    """

    booking_fields: List[BookingFieldCreate]
    confirm_mode: ProductUpdateRequestConfirmMode
    description: str
    short_description: str
    advertised_price: Union[Unset, float] = UNSET
    barcode_output_type: Union[Unset, ProductUpdateRequestBarcodeOutputType] = UNSET
    confirm_mode_min_participants: Union[Unset, int] = UNSET
    duration_minutes: Union[Unset, int] = UNSET
    internal_code: Union[Unset, str] = UNSET
    minimum_notice_minutes: Union[Unset, int] = UNSET
    name: Union[Unset, str] = UNSET
    pickup_id: Union[Unset, int] = UNSET
    price_options: Union[Unset, List[PriceOptionCreate]] = UNSET
    terms: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        booking_fields = []
        for booking_fields_item_data in self.booking_fields:
            booking_fields_item = booking_fields_item_data.to_dict()

            booking_fields.append(booking_fields_item)

        confirm_mode = self.confirm_mode.value

        description = self.description
        short_description = self.short_description
        advertised_price = self.advertised_price
        barcode_output_type: Union[Unset, str] = UNSET
        if not isinstance(self.barcode_output_type, Unset):
            barcode_output_type = self.barcode_output_type.value

        confirm_mode_min_participants = self.confirm_mode_min_participants
        duration_minutes = self.duration_minutes
        internal_code = self.internal_code
        minimum_notice_minutes = self.minimum_notice_minutes
        name = self.name
        pickup_id = self.pickup_id
        price_options: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.price_options, Unset):
            price_options = []
            for price_options_item_data in self.price_options:
                price_options_item = price_options_item_data.to_dict()

                price_options.append(price_options_item)

        terms = self.terms

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "bookingFields": booking_fields,
                "confirmMode": confirm_mode,
                "description": description,
                "shortDescription": short_description,
            }
        )
        if advertised_price is not UNSET:
            field_dict["advertisedPrice"] = advertised_price
        if barcode_output_type is not UNSET:
            field_dict["barcodeOutputType"] = barcode_output_type
        if confirm_mode_min_participants is not UNSET:
            field_dict["confirmModeMinParticipants"] = confirm_mode_min_participants
        if duration_minutes is not UNSET:
            field_dict["durationMinutes"] = duration_minutes
        if internal_code is not UNSET:
            field_dict["internalCode"] = internal_code
        if minimum_notice_minutes is not UNSET:
            field_dict["minimumNoticeMinutes"] = minimum_notice_minutes
        if name is not UNSET:
            field_dict["name"] = name
        if pickup_id is not UNSET:
            field_dict["pickupId"] = pickup_id
        if price_options is not UNSET:
            field_dict["priceOptions"] = price_options
        if terms is not UNSET:
            field_dict["terms"] = terms

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        booking_fields = []
        _booking_fields = d.pop("bookingFields")
        for booking_fields_item_data in _booking_fields:
            booking_fields_item = BookingFieldCreate.from_dict(booking_fields_item_data)

            booking_fields.append(booking_fields_item)

        confirm_mode = ProductUpdateRequestConfirmMode(d.pop("confirmMode"))

        description = d.pop("description")

        short_description = d.pop("shortDescription")

        advertised_price = d.pop("advertisedPrice", UNSET)

        _barcode_output_type = d.pop("barcodeOutputType", UNSET)
        barcode_output_type: Union[Unset, ProductUpdateRequestBarcodeOutputType]
        if isinstance(_barcode_output_type, Unset):
            barcode_output_type = UNSET
        else:
            barcode_output_type = ProductUpdateRequestBarcodeOutputType(_barcode_output_type)

        confirm_mode_min_participants = d.pop("confirmModeMinParticipants", UNSET)

        duration_minutes = d.pop("durationMinutes", UNSET)

        internal_code = d.pop("internalCode", UNSET)

        minimum_notice_minutes = d.pop("minimumNoticeMinutes", UNSET)

        name = d.pop("name", UNSET)

        pickup_id = d.pop("pickupId", UNSET)

        price_options = []
        _price_options = d.pop("priceOptions", UNSET)
        for price_options_item_data in _price_options or []:
            price_options_item = PriceOptionCreate.from_dict(price_options_item_data)

            price_options.append(price_options_item)

        terms = d.pop("terms", UNSET)

        product_update_request = cls(
            booking_fields=booking_fields,
            confirm_mode=confirm_mode,
            description=description,
            short_description=short_description,
            advertised_price=advertised_price,
            barcode_output_type=barcode_output_type,
            confirm_mode_min_participants=confirm_mode_min_participants,
            duration_minutes=duration_minutes,
            internal_code=internal_code,
            minimum_notice_minutes=minimum_notice_minutes,
            name=name,
            pickup_id=pickup_id,
            price_options=price_options,
            terms=terms,
        )

        product_update_request.additional_properties = d
        return product_update_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



""" Contains all the data models used in inputs/outputs """

from .add_product_image_multipart_data import AddProductImageMultipartData
from .address import Address
from .booking import Booking
from .booking_barcode_type import BookingBarcodeType
from .booking_create import BookingCreate
from .booking_create_barcode_type import BookingCreateBarcodeType
from .booking_create_payment_option import BookingCreatePaymentOption
from .booking_create_reseller_source import BookingCreateResellerSource
from .booking_create_source import BookingCreateSource
from .booking_create_status import BookingCreateStatus
from .booking_field import BookingField
from .booking_field_create import BookingFieldCreate
from .booking_field_create_field_type import BookingFieldCreateFieldType
from .booking_field_field_type import BookingFieldFieldType
from .booking_item import BookingItem
from .booking_item_create import BookingItemCreate
from .booking_payment import BookingPayment
from .booking_payment_currency import BookingPaymentCurrency
from .booking_payment_option import BookingPaymentOption
from .booking_payment_recipient import BookingPaymentRecipient
from .booking_payment_type import BookingPaymentType
from .booking_reseller_source import BookingResellerSource
from .booking_source import BookingSource
from .booking_status import BookingStatus
from .booking_total_currency import BookingTotalCurrency
from .booking_update import BookingUpdate
from .booking_update_status import BookingUpdateStatus
from .category import Category
from .category_seo_tag import CategorySeoTag
from .category_seo_tag_meta_type import CategorySeoTagMetaType
from .check_in import CheckIn
from .company import Company
from .credit_card import CreditCard
from .credit_card_card_type import CreditCardCardType
from .customer import Customer
from .customer_gender import CustomerGender
from .customer_title import CustomerTitle
from .error import Error
from .extra import Extra
from .extra_create_request import ExtraCreateRequest
from .extra_create_request_extra_price_type import ExtraCreateRequestExtraPriceType
from .extra_extra_price_type import ExtraExtraPriceType
from .extra_request import ExtraRequest
from .extra_request_extra_price_type import ExtraRequestExtraPriceType
from .form_data_content_disposition import FormDataContentDisposition
from .form_data_content_disposition_parameters import FormDataContentDispositionParameters
from .image import Image
from .net_rate import NetRate
from .participant import Participant
from .pickup_list import PickupList
from .pickup_location import PickupLocation
from .price_option import PriceOption
from .price_option_create import PriceOptionCreate
from .price_option_create_price_group_type import PriceOptionCreatePriceGroupType
from .price_option_price_group_type import PriceOptionPriceGroupType
from .product import Product
from .product_agent_payment_type import ProductAgentPaymentType
from .product_barcode_output_type import ProductBarcodeOutputType
from .product_booking_mode import ProductBookingMode
from .product_confirm_mode import ProductConfirmMode
from .product_create_request import ProductCreateRequest
from .product_create_request_barcode_output_type import ProductCreateRequestBarcodeOutputType
from .product_create_request_booking_mode import ProductCreateRequestBookingMode
from .product_create_request_confirm_mode import ProductCreateRequestConfirmMode
from .product_create_request_product_type import ProductCreateRequestProductType
from .product_currency import ProductCurrency
from .product_product_type import ProductProductType
from .product_qr_code_type import ProductQrCodeType
from .product_rate import ProductRate
from .product_rate_commission_type import ProductRateCommissionType
from .product_seo_tag import ProductSeoTag
from .product_seo_tag_meta_type import ProductSeoTagMetaType
from .product_update_request import ProductUpdateRequest
from .product_update_request_barcode_output_type import ProductUpdateRequestBarcodeOutputType
from .product_update_request_confirm_mode import ProductUpdateRequestConfirmMode
from .quantity import Quantity
from .rate import Rate
from .request_status import RequestStatus
from .resource import Resource
from .resource_type import ResourceType
from .response_booking import ResponseBooking
from .response_booking_list import ResponseBookingList
from .response_category import ResponseCategory
from .response_category_list import ResponseCategoryList
from .response_check_in import ResponseCheckIn
from .response_company import ResponseCompany
from .response_customer import ResponseCustomer
from .response_customer_list import ResponseCustomerList
from .response_extra import ResponseExtra
from .response_extra_list import ResponseExtraList
from .response_image import ResponseImage
from .response_no_data import ResponseNoData
from .response_pickup_list import ResponsePickupList
from .response_pickup_list_list import ResponsePickupListList
from .response_pickup_location_list import ResponsePickupLocationList
from .response_product import ResponseProduct
from .response_product_list import ResponseProductList
from .response_rate import ResponseRate
from .response_rate_list import ResponseRateList
from .response_resource import ResponseResource
from .response_resource_list import ResponseResourceList
from .response_rezdy_connect_settings import ResponseRezdyConnectSettings
from .response_session import ResponseSession
from .response_session_list import ResponseSessionList
from .response_voucher import ResponseVoucher
from .response_voucher_list import ResponseVoucherList
from .rezdy_connect_settings import RezdyConnectSettings
from .rezdy_connect_settings_external_api_format import RezdyConnectSettingsExternalApiFormat
from .rezdy_connect_settings_external_booking_strategy import RezdyConnectSettingsExternalBookingStrategy
from .search_bookings_order_status import SearchBookingsOrderStatus
from .session import Session
from .session_create_request import SessionCreateRequest
from .session_update_batch_request import SessionUpdateBatchRequest
from .session_update_request import SessionUpdateRequest
from .tax import Tax
from .tax_tax_fee_type import TaxTaxFeeType
from .tax_tax_type import TaxTaxType
from .user import User
from .video import Video
from .voucher import Voucher
from .voucher_status import VoucherStatus
from .voucher_value_type import VoucherValueType
from .warning import Warning_



from enum import Enum


class ProductCreateRequestConfirmMode(str, Enum):
    MANUAL = "MANUAL"
    AUTOCONFIRM = "AUTOCONFIRM"
    MANUAL_THEN_AUTO = "MANUAL_THEN_AUTO"
    AUTO_THEN_MANUAL = "AUTO_THEN_MANUAL"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class VoucherValueType(str, Enum):
    VALUE_LIMITPRODUCT = "VALUE_LIMITPRODUCT"
    VALUE = "VALUE"
    VALUE_LIMITCATALOG = "VALUE_LIMITCATALOG"
    PERCENT_LIMITPRODUCT = "PERCENT_LIMITPRODUCT"
    PERCENT = "PERCENT"
    PERCENT_LIMITCATALOG = "PERCENT_LIMITCATALOG"
    PRODUCT = "PRODUCT"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class TaxTaxType(str, Enum):
    PERCENT = "PERCENT"
    FIXED_PER_QUANTITY = "FIXED_PER_QUANTITY"
    FIXED_PER_ORDER = "FIXED_PER_ORDER"
    FIXED_PER_DURATION = "FIXED_PER_DURATION"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.address import Address
from ..models.video import Video
from ..types import UNSET, Unset

T = TypeVar("T", bound="Company")


@attr.s(auto_attribs=True)
class Company:
    """Company object. Holds general details and information about a specific company.

    Attributes:
        address (Union[Unset, Address]): Address of a company, customer or product location.
        agent_description (Union[Unset, str]): Agent description, if the company is an agent.
        agent_registration_link (Union[Unset, str]): Agent registration link, if the company is an agent
        alias (Union[Unset, str]): Company alias. This is the unique identifier for this company
        booking_system (Union[Unset, str]): Company Booking System
        category (Union[Unset, str]): Company category
        company_description (Union[Unset, str]): Company description
        company_logo_url (Union[Unset, str]): URL of the company logo.
        company_name (Union[Unset, str]): Company name
        currency (Union[Unset, str]): Default currency used by this company.
        destination_country_code (Union[Unset, str]): Company destination. Country code.
        destination_name (Union[Unset, str]): Company destination. Name of the area.
        destination_path (Union[Unset, str]): Company destination. Destination path.
        facebook_page (Union[Unset, str]): Company facebook page
        fax (Union[Unset, str]): Company fax
        fb_page_id (Union[Unset, str]): Company facebook page id
        first_name (Union[Unset, str]): First name of company representative
        google_plus (Union[Unset, str]): Company google plus profile
        instagram (Union[Unset, str]): Company instagram profile
        last_name (Union[Unset, str]): Last name of company representative
        locale (Union[Unset, str]): Locale of this company.
        mobile (Union[Unset, str]): Company mobile
        opening_hours (Union[Unset, str]): Company opening hours
        phone (Union[Unset, str]): Company phone
        pinterest (Union[Unset, str]): Company pinterest profile
        privacy_policy (Union[Unset, str]): Privacy Policy
        skype (Union[Unset, str]): Company skype
        terms (Union[Unset, str]): General terms and conditions
        timezone (Union[Unset, str]): Timezone of this company.
        trip_advisor (Union[Unset, str]): Company trip advisor profile
        twitter (Union[Unset, str]): Company trip twitter profile
        video (Union[Unset, Video]): Video links.
        website (Union[Unset, str]): Company website
        yelp (Union[Unset, str]): Company yelp profile
        youtube_channel (Union[Unset, str]): Company youtube channel
    """

    address: Union[Unset, Address] = UNSET
    agent_description: Union[Unset, str] = UNSET
    agent_registration_link: Union[Unset, str] = UNSET
    alias: Union[Unset, str] = UNSET
    booking_system: Union[Unset, str] = UNSET
    category: Union[Unset, str] = UNSET
    company_description: Union[Unset, str] = UNSET
    company_logo_url: Union[Unset, str] = UNSET
    company_name: Union[Unset, str] = UNSET
    currency: Union[Unset, str] = UNSET
    destination_country_code: Union[Unset, str] = UNSET
    destination_name: Union[Unset, str] = UNSET
    destination_path: Union[Unset, str] = UNSET
    facebook_page: Union[Unset, str] = UNSET
    fax: Union[Unset, str] = UNSET
    fb_page_id: Union[Unset, str] = UNSET
    first_name: Union[Unset, str] = UNSET
    google_plus: Union[Unset, str] = UNSET
    instagram: Union[Unset, str] = UNSET
    last_name: Union[Unset, str] = UNSET
    locale: Union[Unset, str] = UNSET
    mobile: Union[Unset, str] = UNSET
    opening_hours: Union[Unset, str] = UNSET
    phone: Union[Unset, str] = UNSET
    pinterest: Union[Unset, str] = UNSET
    privacy_policy: Union[Unset, str] = UNSET
    skype: Union[Unset, str] = UNSET
    terms: Union[Unset, str] = UNSET
    timezone: Union[Unset, str] = UNSET
    trip_advisor: Union[Unset, str] = UNSET
    twitter: Union[Unset, str] = UNSET
    video: Union[Unset, Video] = UNSET
    website: Union[Unset, str] = UNSET
    yelp: Union[Unset, str] = UNSET
    youtube_channel: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        address: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.address, Unset):
            address = self.address.to_dict()

        agent_description = self.agent_description
        agent_registration_link = self.agent_registration_link
        alias = self.alias
        booking_system = self.booking_system
        category = self.category
        company_description = self.company_description
        company_logo_url = self.company_logo_url
        company_name = self.company_name
        currency = self.currency
        destination_country_code = self.destination_country_code
        destination_name = self.destination_name
        destination_path = self.destination_path
        facebook_page = self.facebook_page
        fax = self.fax
        fb_page_id = self.fb_page_id
        first_name = self.first_name
        google_plus = self.google_plus
        instagram = self.instagram
        last_name = self.last_name
        locale = self.locale
        mobile = self.mobile
        opening_hours = self.opening_hours
        phone = self.phone
        pinterest = self.pinterest
        privacy_policy = self.privacy_policy
        skype = self.skype
        terms = self.terms
        timezone = self.timezone
        trip_advisor = self.trip_advisor
        twitter = self.twitter
        video: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.video, Unset):
            video = self.video.to_dict()

        website = self.website
        yelp = self.yelp
        youtube_channel = self.youtube_channel

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if address is not UNSET:
            field_dict["address"] = address
        if agent_description is not UNSET:
            field_dict["agentDescription"] = agent_description
        if agent_registration_link is not UNSET:
            field_dict["agentRegistrationLink"] = agent_registration_link
        if alias is not UNSET:
            field_dict["alias"] = alias
        if booking_system is not UNSET:
            field_dict["bookingSystem"] = booking_system
        if category is not UNSET:
            field_dict["category"] = category
        if company_description is not UNSET:
            field_dict["companyDescription"] = company_description
        if company_logo_url is not UNSET:
            field_dict["companyLogoUrl"] = company_logo_url
        if company_name is not UNSET:
            field_dict["companyName"] = company_name
        if currency is not UNSET:
            field_dict["currency"] = currency
        if destination_country_code is not UNSET:
            field_dict["destinationCountryCode"] = destination_country_code
        if destination_name is not UNSET:
            field_dict["destinationName"] = destination_name
        if destination_path is not UNSET:
            field_dict["destinationPath"] = destination_path
        if facebook_page is not UNSET:
            field_dict["facebookPage"] = facebook_page
        if fax is not UNSET:
            field_dict["fax"] = fax
        if fb_page_id is not UNSET:
            field_dict["fbPageId"] = fb_page_id
        if first_name is not UNSET:
            field_dict["firstName"] = first_name
        if google_plus is not UNSET:
            field_dict["googlePlus"] = google_plus
        if instagram is not UNSET:
            field_dict["instagram"] = instagram
        if last_name is not UNSET:
            field_dict["lastName"] = last_name
        if locale is not UNSET:
            field_dict["locale"] = locale
        if mobile is not UNSET:
            field_dict["mobile"] = mobile
        if opening_hours is not UNSET:
            field_dict["openingHours"] = opening_hours
        if phone is not UNSET:
            field_dict["phone"] = phone
        if pinterest is not UNSET:
            field_dict["pinterest"] = pinterest
        if privacy_policy is not UNSET:
            field_dict["privacyPolicy"] = privacy_policy
        if skype is not UNSET:
            field_dict["skype"] = skype
        if terms is not UNSET:
            field_dict["terms"] = terms
        if timezone is not UNSET:
            field_dict["timezone"] = timezone
        if trip_advisor is not UNSET:
            field_dict["tripAdvisor"] = trip_advisor
        if twitter is not UNSET:
            field_dict["twitter"] = twitter
        if video is not UNSET:
            field_dict["video"] = video
        if website is not UNSET:
            field_dict["website"] = website
        if yelp is not UNSET:
            field_dict["yelp"] = yelp
        if youtube_channel is not UNSET:
            field_dict["youtubeChannel"] = youtube_channel

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _address = d.pop("address", UNSET)
        address: Union[Unset, Address]
        if isinstance(_address, Unset):
            address = UNSET
        else:
            address = Address.from_dict(_address)

        agent_description = d.pop("agentDescription", UNSET)

        agent_registration_link = d.pop("agentRegistrationLink", UNSET)

        alias = d.pop("alias", UNSET)

        booking_system = d.pop("bookingSystem", UNSET)

        category = d.pop("category", UNSET)

        company_description = d.pop("companyDescription", UNSET)

        company_logo_url = d.pop("companyLogoUrl", UNSET)

        company_name = d.pop("companyName", UNSET)

        currency = d.pop("currency", UNSET)

        destination_country_code = d.pop("destinationCountryCode", UNSET)

        destination_name = d.pop("destinationName", UNSET)

        destination_path = d.pop("destinationPath", UNSET)

        facebook_page = d.pop("facebookPage", UNSET)

        fax = d.pop("fax", UNSET)

        fb_page_id = d.pop("fbPageId", UNSET)

        first_name = d.pop("firstName", UNSET)

        google_plus = d.pop("googlePlus", UNSET)

        instagram = d.pop("instagram", UNSET)

        last_name = d.pop("lastName", UNSET)

        locale = d.pop("locale", UNSET)

        mobile = d.pop("mobile", UNSET)

        opening_hours = d.pop("openingHours", UNSET)

        phone = d.pop("phone", UNSET)

        pinterest = d.pop("pinterest", UNSET)

        privacy_policy = d.pop("privacyPolicy", UNSET)

        skype = d.pop("skype", UNSET)

        terms = d.pop("terms", UNSET)

        timezone = d.pop("timezone", UNSET)

        trip_advisor = d.pop("tripAdvisor", UNSET)

        twitter = d.pop("twitter", UNSET)

        _video = d.pop("video", UNSET)
        video: Union[Unset, Video]
        if isinstance(_video, Unset):
            video = UNSET
        else:
            video = Video.from_dict(_video)

        website = d.pop("website", UNSET)

        yelp = d.pop("yelp", UNSET)

        youtube_channel = d.pop("youtubeChannel", UNSET)

        company = cls(
            address=address,
            agent_description=agent_description,
            agent_registration_link=agent_registration_link,
            alias=alias,
            booking_system=booking_system,
            category=category,
            company_description=company_description,
            company_logo_url=company_logo_url,
            company_name=company_name,
            currency=currency,
            destination_country_code=destination_country_code,
            destination_name=destination_name,
            destination_path=destination_path,
            facebook_page=facebook_page,
            fax=fax,
            fb_page_id=fb_page_id,
            first_name=first_name,
            google_plus=google_plus,
            instagram=instagram,
            last_name=last_name,
            locale=locale,
            mobile=mobile,
            opening_hours=opening_hours,
            phone=phone,
            pinterest=pinterest,
            privacy_policy=privacy_policy,
            skype=skype,
            terms=terms,
            timezone=timezone,
            trip_advisor=trip_advisor,
            twitter=twitter,
            video=video,
            website=website,
            yelp=yelp,
            youtube_channel=youtube_channel,
        )

        company.additional_properties = d
        return company

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class RezdyConnectSettingsExternalBookingStrategy(str, Enum):
    TWO_STEPS_BOOKING = "TWO_STEPS_BOOKING"
    ONE_STEP_BOOKING = "ONE_STEP_BOOKING"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class BookingFieldFieldType(str, Enum):
    STRING = "String"
    LIST = "List"
    BOOLEAN = "Boolean"
    PHONE = "Phone"
    DATE = "Date"
    HIDDEN = "Hidden"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="Quantity")


@attr.s(auto_attribs=True)
class Quantity:
    """Quantity of a single price option attached to a BookingItem.<ul><li>If the product only has 1 price option, only
    "Quantity.value" is required.</li><li>If the product has multiple price options, "Quantity.optionLabel" is also
    required.</li><li>It is recommended to use "Quantity.optionLabel" and optionally "Quantity.optionPrice" instead of
    "Quantity.optionId" because the latter can vary depending on the session booked.</li></ul>I.e. enter optionLabel =
    "Adult", optionPrice = 100 and value = "2" to book for 2 x Adults ticket for 100

        Attributes:
            value (int): Quantity actually booked
            option_id (Union[Unset, int]): Price option ID
            option_label (Union[Unset, str]): Price option label
            option_price (Union[Unset, float]): Price option price for a single quantity
    """

    value: int
    option_id: Union[Unset, int] = UNSET
    option_label: Union[Unset, str] = UNSET
    option_price: Union[Unset, float] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        value = self.value
        option_id = self.option_id
        option_label = self.option_label
        option_price = self.option_price

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "value": value,
            }
        )
        if option_id is not UNSET:
            field_dict["optionId"] = option_id
        if option_label is not UNSET:
            field_dict["optionLabel"] = option_label
        if option_price is not UNSET:
            field_dict["optionPrice"] = option_price

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        value = d.pop("value")

        option_id = d.pop("optionId", UNSET)

        option_label = d.pop("optionLabel", UNSET)

        option_price = d.pop("optionPrice", UNSET)

        quantity = cls(
            value=value,
            option_id=option_id,
            option_label=option_label,
            option_price=option_price,
        )

        quantity.additional_properties = d
        return quantity

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.credit_card_card_type import CreditCardCardType
from ..types import UNSET, Unset

T = TypeVar("T", bound="CreditCard")


@attr.s(auto_attribs=True)
class CreditCard:
    """Credit card details.<p>Used to send payment details for a booking</p>

    Attributes:
        card_country (Union[Unset, str]):
        card_name (Union[Unset, str]): Full name on the credit card
        card_number (Union[Unset, str]): Full Credit card number, no spaces or special characters
        card_security_number (Union[Unset, str]): Credit Card CVV / CVN
        card_token (Union[Unset, str]):  Credit card Token generated by client libraries. Use instead of other card
            attributes for supported gateways.<br>For example when using Stripe.js, only send the cardToken attribute. Do
            not send cardNumber or any other attribute.
        card_type (Union[Unset, CreditCardCardType]): Type of card
        expiry_month (Union[Unset, str]): <ul>Expiry month of the card<li>01 : January</li><li>02 : February</li><li>03
            : March</li><li>04 : April</li><li>05 : May</li><li>06 : June</li><li>07 : July</li><li>08 : August</li><li>09 :
            September</li><li>10 : October</li><li>11 : November</li><li>12 : December</li></ul>
        expiry_year (Union[Unset, str]): Expiry year of the card in 4 digits<br>(i.e 2015 / 2016 ...)
    """

    card_country: Union[Unset, str] = UNSET
    card_name: Union[Unset, str] = UNSET
    card_number: Union[Unset, str] = UNSET
    card_security_number: Union[Unset, str] = UNSET
    card_token: Union[Unset, str] = UNSET
    card_type: Union[Unset, CreditCardCardType] = UNSET
    expiry_month: Union[Unset, str] = UNSET
    expiry_year: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        card_country = self.card_country
        card_name = self.card_name
        card_number = self.card_number
        card_security_number = self.card_security_number
        card_token = self.card_token
        card_type: Union[Unset, str] = UNSET
        if not isinstance(self.card_type, Unset):
            card_type = self.card_type.value

        expiry_month = self.expiry_month
        expiry_year = self.expiry_year

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if card_country is not UNSET:
            field_dict["cardCountry"] = card_country
        if card_name is not UNSET:
            field_dict["cardName"] = card_name
        if card_number is not UNSET:
            field_dict["cardNumber"] = card_number
        if card_security_number is not UNSET:
            field_dict["cardSecurityNumber"] = card_security_number
        if card_token is not UNSET:
            field_dict["cardToken"] = card_token
        if card_type is not UNSET:
            field_dict["cardType"] = card_type
        if expiry_month is not UNSET:
            field_dict["expiryMonth"] = expiry_month
        if expiry_year is not UNSET:
            field_dict["expiryYear"] = expiry_year

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        card_country = d.pop("cardCountry", UNSET)

        card_name = d.pop("cardName", UNSET)

        card_number = d.pop("cardNumber", UNSET)

        card_security_number = d.pop("cardSecurityNumber", UNSET)

        card_token = d.pop("cardToken", UNSET)

        _card_type = d.pop("cardType", UNSET)
        card_type: Union[Unset, CreditCardCardType]
        if isinstance(_card_type, Unset):
            card_type = UNSET
        else:
            card_type = CreditCardCardType(_card_type)

        expiry_month = d.pop("expiryMonth", UNSET)

        expiry_year = d.pop("expiryYear", UNSET)

        credit_card = cls(
            card_country=card_country,
            card_name=card_name,
            card_number=card_number,
            card_security_number=card_security_number,
            card_token=card_token,
            card_type=card_type,
            expiry_month=expiry_month,
            expiry_year=expiry_year,
        )

        credit_card.additional_properties = d
        return credit_card

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.extra_create_request_extra_price_type import ExtraCreateRequestExtraPriceType
from ..types import UNSET, Unset

T = TypeVar("T", bound="ExtraCreateRequest")


@attr.s(auto_attribs=True)
class ExtraCreateRequest:
    """Partial optional service or item that can be purchased when booking a specific product

    Attributes:
        description (Union[Unset, str]): Description of the extra
        extra_price_type (Union[Unset, ExtraCreateRequestExtraPriceType]): Price type for this extra. Defines what
            quantities are allowed and how their price is calculated
        name (Union[Unset, str]): Name of the extra
        price (Union[Unset, float]): Price for a single quantity of this extra
    """

    description: Union[Unset, str] = UNSET
    extra_price_type: Union[Unset, ExtraCreateRequestExtraPriceType] = UNSET
    name: Union[Unset, str] = UNSET
    price: Union[Unset, float] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        description = self.description
        extra_price_type: Union[Unset, str] = UNSET
        if not isinstance(self.extra_price_type, Unset):
            extra_price_type = self.extra_price_type.value

        name = self.name
        price = self.price

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if description is not UNSET:
            field_dict["description"] = description
        if extra_price_type is not UNSET:
            field_dict["extraPriceType"] = extra_price_type
        if name is not UNSET:
            field_dict["name"] = name
        if price is not UNSET:
            field_dict["price"] = price

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        description = d.pop("description", UNSET)

        _extra_price_type = d.pop("extraPriceType", UNSET)
        extra_price_type: Union[Unset, ExtraCreateRequestExtraPriceType]
        if isinstance(_extra_price_type, Unset):
            extra_price_type = UNSET
        else:
            extra_price_type = ExtraCreateRequestExtraPriceType(_extra_price_type)

        name = d.pop("name", UNSET)

        price = d.pop("price", UNSET)

        extra_create_request = cls(
            description=description,
            extra_price_type=extra_price_type,
            name=name,
            price=price,
        )

        extra_create_request.additional_properties = d
        return extra_create_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.tax_tax_fee_type import TaxTaxFeeType
from ..models.tax_tax_type import TaxTaxType
from ..types import UNSET, Unset

T = TypeVar("T", bound="Tax")


@attr.s(auto_attribs=True)
class Tax:
    """Tax object. Holds information such as the tax amount applied to an order

    Attributes:
        compound (Union[Unset, bool]): Whether a stacked tax with the specified percent is applied. <br> e.g. A $100
            item with an exclusive tax of %10 will result in the price being $110. If compound is true, then an addition %10
            of $110 will be added as tax.
        label (Union[Unset, str]): Name/description of the tax
        price_inclusive (Union[Unset, bool]): Whether the tax is included in the price or not. This field will be
            displayed if the taxType is PERCENT <br>E.g. A $100 item with price INCLUSIVE tax of 10% will result in a $10
            tax as part of the $100. A $100 item with price EXCLUSIVE tax of 10% will result in a $10 tax on top of the
            $100.
        supplier_id (Union[Unset, int]): Rezdy internal ID of the company applying this tax
        tax_amount (Union[Unset, float]): The tax amount. This field will only contain a value if the taxType is one of
            the following: FIXED_PER_QUANTITY, FIXED_PER_ORDER, FIXED_PER_DURATION
        tax_fee_type (Union[Unset, TaxTaxFeeType]): Indicate Fee or Tax
        tax_percent (Union[Unset, float]): Percentage value of the fee/tax. This field will only contain a value if the
            taxType is PERCENT
        tax_type (Union[Unset, TaxTaxType]): <b>PERCENT: </b>The tax will be a percentage of the order
            total.<br><b>FIXED_PER_QUANTITY: </b>The tax will be a fixed amount e.g. $10 per
            quantity.<br><b>FIXED_PER_ORDER: </b>The tax will be a fixed amount e.g. $10 per booking
            item.<br><b>FIXED_PER_DURATION: </b>The tax will be a fixed amount e.g. $10 per duration unit.<br>
    """

    compound: Union[Unset, bool] = UNSET
    label: Union[Unset, str] = UNSET
    price_inclusive: Union[Unset, bool] = UNSET
    supplier_id: Union[Unset, int] = UNSET
    tax_amount: Union[Unset, float] = UNSET
    tax_fee_type: Union[Unset, TaxTaxFeeType] = UNSET
    tax_percent: Union[Unset, float] = UNSET
    tax_type: Union[Unset, TaxTaxType] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        compound = self.compound
        label = self.label
        price_inclusive = self.price_inclusive
        supplier_id = self.supplier_id
        tax_amount = self.tax_amount
        tax_fee_type: Union[Unset, str] = UNSET
        if not isinstance(self.tax_fee_type, Unset):
            tax_fee_type = self.tax_fee_type.value

        tax_percent = self.tax_percent
        tax_type: Union[Unset, str] = UNSET
        if not isinstance(self.tax_type, Unset):
            tax_type = self.tax_type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if compound is not UNSET:
            field_dict["compound"] = compound
        if label is not UNSET:
            field_dict["label"] = label
        if price_inclusive is not UNSET:
            field_dict["priceInclusive"] = price_inclusive
        if supplier_id is not UNSET:
            field_dict["supplierId"] = supplier_id
        if tax_amount is not UNSET:
            field_dict["taxAmount"] = tax_amount
        if tax_fee_type is not UNSET:
            field_dict["taxFeeType"] = tax_fee_type
        if tax_percent is not UNSET:
            field_dict["taxPercent"] = tax_percent
        if tax_type is not UNSET:
            field_dict["taxType"] = tax_type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        compound = d.pop("compound", UNSET)

        label = d.pop("label", UNSET)

        price_inclusive = d.pop("priceInclusive", UNSET)

        supplier_id = d.pop("supplierId", UNSET)

        tax_amount = d.pop("taxAmount", UNSET)

        _tax_fee_type = d.pop("taxFeeType", UNSET)
        tax_fee_type: Union[Unset, TaxTaxFeeType]
        if isinstance(_tax_fee_type, Unset):
            tax_fee_type = UNSET
        else:
            tax_fee_type = TaxTaxFeeType(_tax_fee_type)

        tax_percent = d.pop("taxPercent", UNSET)

        _tax_type = d.pop("taxType", UNSET)
        tax_type: Union[Unset, TaxTaxType]
        if isinstance(_tax_type, Unset):
            tax_type = UNSET
        else:
            tax_type = TaxTaxType(_tax_type)

        tax = cls(
            compound=compound,
            label=label,
            price_inclusive=price_inclusive,
            supplier_id=supplier_id,
            tax_amount=tax_amount,
            tax_fee_type=tax_fee_type,
            tax_percent=tax_percent,
            tax_type=tax_type,
        )

        tax.additional_properties = d
        return tax

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.voucher_status import VoucherStatus
from ..models.voucher_value_type import VoucherValueType
from ..types import UNSET, Unset

T = TypeVar("T", bound="Voucher")


@attr.s(auto_attribs=True)
class Voucher:
    """A Voucher is a gift card that can be used to pay for a booking.<br>It can also be generated by a booking when a
    customer buys a Gift Card.<br>This is not a "confirmation voucher", you should use Booking.orderNumber as a
    confirmation code to show to customers.

        Attributes:
            code (Union[Unset, str]): Unique voucher code
            expiry_date (Union[Unset, datetime.datetime]): Date this voucher will expire
            internal_notes (Union[Unset, str]): Internal notes only visible by the supplier of this voucher
            internal_reference (Union[Unset, str]): Agent code attached to this vouche
            issue_date (Union[Unset, datetime.datetime]): Date this voucher was created
            source_order (Union[Unset, str]): If the voucher was generated from an order - by purchasing a gift card or
                product as a gift card, it will contain the orderNumber which created the voucher. Null for vouchers manually
                created by user.
            status (Union[Unset, VoucherStatus]): Current voucher status
            value (Union[Unset, float]): The value of the voucher. There are different representations of the value -
                nominal, percentage or empty - in case of the payment for the whole product, see valueType
            value_type (Union[Unset, VoucherValueType]): <dl><dt>VALUE_LIMITPRODUCT</dt><dd>Voucher nominal value discount,
                limited to a certain product</dd><dt>VALUE</dt><dd>Voucher nominal value discount for any
                product</dd><dt>VALUE_LIMITCATALOG</dt><dd>Voucher nominal value discount, limited to a certain product
                category</dd><dt>PERCENT_LIMITPRODUCT</dt><dd>Voucher percentage value discount, limited to a certain
                product</dd><dt>PERCENT</dt><dd>Voucher percentage value discount for any
                product</dd><dt>PERCENT_LIMITCATALOG</dt><dd>Voucher percentage value discount, limited to a certain product
                category</dd><dt>PRODUCT</dt><dd>Voucher can be used as a payment method for a specific product</dd></dl>
    """

    code: Union[Unset, str] = UNSET
    expiry_date: Union[Unset, datetime.datetime] = UNSET
    internal_notes: Union[Unset, str] = UNSET
    internal_reference: Union[Unset, str] = UNSET
    issue_date: Union[Unset, datetime.datetime] = UNSET
    source_order: Union[Unset, str] = UNSET
    status: Union[Unset, VoucherStatus] = UNSET
    value: Union[Unset, float] = UNSET
    value_type: Union[Unset, VoucherValueType] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        code = self.code
        expiry_date: Union[Unset, str] = UNSET
        if not isinstance(self.expiry_date, Unset):
            expiry_date = self.expiry_date.isoformat()

        internal_notes = self.internal_notes
        internal_reference = self.internal_reference
        issue_date: Union[Unset, str] = UNSET
        if not isinstance(self.issue_date, Unset):
            issue_date = self.issue_date.isoformat()

        source_order = self.source_order
        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        value = self.value
        value_type: Union[Unset, str] = UNSET
        if not isinstance(self.value_type, Unset):
            value_type = self.value_type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if code is not UNSET:
            field_dict["code"] = code
        if expiry_date is not UNSET:
            field_dict["expiryDate"] = expiry_date
        if internal_notes is not UNSET:
            field_dict["internalNotes"] = internal_notes
        if internal_reference is not UNSET:
            field_dict["internalReference"] = internal_reference
        if issue_date is not UNSET:
            field_dict["issueDate"] = issue_date
        if source_order is not UNSET:
            field_dict["sourceOrder"] = source_order
        if status is not UNSET:
            field_dict["status"] = status
        if value is not UNSET:
            field_dict["value"] = value
        if value_type is not UNSET:
            field_dict["valueType"] = value_type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        code = d.pop("code", UNSET)

        _expiry_date = d.pop("expiryDate", UNSET)
        expiry_date: Union[Unset, datetime.datetime]
        if isinstance(_expiry_date, Unset):
            expiry_date = UNSET
        else:
            expiry_date = isoparse(_expiry_date)

        internal_notes = d.pop("internalNotes", UNSET)

        internal_reference = d.pop("internalReference", UNSET)

        _issue_date = d.pop("issueDate", UNSET)
        issue_date: Union[Unset, datetime.datetime]
        if isinstance(_issue_date, Unset):
            issue_date = UNSET
        else:
            issue_date = isoparse(_issue_date)

        source_order = d.pop("sourceOrder", UNSET)

        _status = d.pop("status", UNSET)
        status: Union[Unset, VoucherStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = VoucherStatus(_status)

        value = d.pop("value", UNSET)

        _value_type = d.pop("valueType", UNSET)
        value_type: Union[Unset, VoucherValueType]
        if isinstance(_value_type, Unset):
            value_type = UNSET
        else:
            value_type = VoucherValueType(_value_type)

        voucher = cls(
            code=code,
            expiry_date=expiry_date,
            internal_notes=internal_notes,
            internal_reference=internal_reference,
            issue_date=issue_date,
            source_order=source_order,
            status=status,
            value=value,
            value_type=value_type,
        )

        voucher.additional_properties = d
        return voucher

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import json
from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.form_data_content_disposition import FormDataContentDisposition
from ..types import UNSET, Unset

T = TypeVar("T", bound="AddProductImageMultipartData")


@attr.s(auto_attribs=True)
class AddProductImageMultipartData:
    """
    Attributes:
        file (FormDataContentDisposition):
        filename (Union[Unset, str]):
    """

    file: FormDataContentDisposition
    filename: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        file = self.file.to_dict()

        filename = self.filename

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "file": file,
            }
        )
        if filename is not UNSET:
            field_dict["filename"] = filename

        return field_dict

    def to_multipart(self) -> Dict[str, Any]:
        file = (None, json.dumps(self.file.to_dict()).encode(), "application/json")

        filename = (
            self.filename if isinstance(self.filename, Unset) else (None, str(self.filename).encode(), "text/plain")
        )

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {key: (None, str(value).encode(), "text/plain") for key, value in self.additional_properties.items()}
        )
        field_dict.update(
            {
                "file": file,
            }
        )
        if filename is not UNSET:
            field_dict["filename"] = filename

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        file = FormDataContentDisposition.from_dict(d.pop("file"))

        filename = d.pop("filename", UNSET)

        add_product_image_multipart_data = cls(
            file=file,
            filename=filename,
        )

        add_product_image_multipart_data.additional_properties = d
        return add_product_image_multipart_data

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict

import httpx

from ...client import Client
from ...models.session_create_request import SessionCreateRequest
from ...types import Response


def _get_kwargs(
    *,
    client: Client,
    json_body: SessionCreateRequest,
) -> Dict[str, Any]:
    url = "{}/availability".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    json_json_body = json_body.to_dict()

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
    }


def _build_response(*, response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=None,
    )


def sync_detailed(
    *,
    client: Client,
    json_body: SessionCreateRequest,
) -> Response[Any]:
    """Create availability

     Create a new session - creates availability for a specific startTime. Sessions can be created only
    for INVENTORY mode
    products.<br>
    The endTime, if not specified, will be calculated based on selected product default duration, and
    the current product
    price options will be used (if not specified). Does not support resources yet<br>
    <p>Response is the newly created Session</p>

    Args:
        json_body (SessionCreateRequest): Create session request data.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


async def asyncio_detailed(
    *,
    client: Client,
    json_body: SessionCreateRequest,
) -> Response[Any]:
    """Create availability

     Create a new session - creates availability for a specific startTime. Sessions can be created only
    for INVENTORY mode
    products.<br>
    The endTime, if not specified, will be calculated based on selected product default duration, and
    the current product
    price options will be used (if not specified). Does not support resources yet<br>
    <p>Response is the newly created Session</p>

    Args:
        json_body (SessionCreateRequest): Create session request data.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)



from enum import Enum


class ProductCreateRequestProductType(str, Enum):
    ACTIVITY = "ACTIVITY"
    DAYTOUR = "DAYTOUR"
    MULTIDAYTOUR = "MULTIDAYTOUR"
    PRIVATE_TOUR = "PRIVATE_TOUR"
    TICKET = "TICKET"
    RENTAL = "RENTAL"
    CHARTER = "CHARTER"
    EVENT = "EVENT"
    GIFT_CARD = "GIFT_CARD"
    TRANSFER = "TRANSFER"
    LESSON = "LESSON"
    MERCHANDISE = "MERCHANDISE"
    CUSTOM = "CUSTOM"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="NetRate")


@attr.s(auto_attribs=True)
class NetRate:
    """A Product with its associated price options net rates

    Attributes:
        net_price (Union[Unset, float]): Value of the rate for the given price option label
        price_option_label (Union[Unset, str]): Label of the price option e.g. Adult, Child etc
    """

    net_price: Union[Unset, float] = UNSET
    price_option_label: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        net_price = self.net_price
        price_option_label = self.price_option_label

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if net_price is not UNSET:
            field_dict["netPrice"] = net_price
        if price_option_label is not UNSET:
            field_dict["priceOptionLabel"] = price_option_label

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        net_price = d.pop("netPrice", UNSET)

        price_option_label = d.pop("priceOptionLabel", UNSET)

        net_rate = cls(
            net_price=net_price,
            price_option_label=price_option_label,
        )

        net_rate.additional_properties = d
        return net_rate

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class BookingPaymentOption(str, Enum):
    CREDITCARD = "CREDITCARD"
    PAYPAL = "PAYPAL"
    BANKTRANSFER = "BANKTRANSFER"
    CASH = "CASH"
    INVOICE = "INVOICE"
    EXTERNAL = "EXTERNAL"
    ALIPAY = "ALIPAY"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.error import Error
from ..models.warning import Warning_
from ..types import UNSET, Unset

T = TypeVar("T", bound="RequestStatus")


@attr.s(auto_attribs=True)
class RequestStatus:
    """
    Attributes:
        success (bool):
        error (Union[Unset, Error]):
        version (Union[Unset, str]):
        warning (Union[Unset, Warning_]):
    """

    success: bool
    error: Union[Unset, Error] = UNSET
    version: Union[Unset, str] = UNSET
    warning: Union[Unset, Warning_] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        success = self.success
        error: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.error, Unset):
            error = self.error.to_dict()

        version = self.version
        warning: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.warning, Unset):
            warning = self.warning.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "success": success,
            }
        )
        if error is not UNSET:
            field_dict["error"] = error
        if version is not UNSET:
            field_dict["version"] = version
        if warning is not UNSET:
            field_dict["warning"] = warning

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        success = d.pop("success")

        _error = d.pop("error", UNSET)
        error: Union[Unset, Error]
        if isinstance(_error, Unset):
            error = UNSET
        else:
            error = Error.from_dict(_error)

        version = d.pop("version", UNSET)

        _warning = d.pop("warning", UNSET)
        warning: Union[Unset, Warning_]
        if isinstance(_warning, Unset):
            warning = UNSET
        else:
            warning = Warning_.from_dict(_warning)

        request_status = cls(
            success=success,
            error=error,
            version=version,
            warning=warning,
        )

        request_status.additional_properties = d
        return request_status

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class CategorySeoTagMetaType(str, Enum):
    META_NAME = "META_NAME"
    META_PROPERTY = "META_PROPERTY"
    LINK_REL = "LINK_REL"
    TITLE = "TITLE"
    NOINDEX = "NOINDEX"
    REL_CANONICAL = "REL_CANONICAL"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="CheckIn")


@attr.s(auto_attribs=True)
class CheckIn:
    """Check-in information.

    Attributes:
        is_checke_in (Union[Unset, bool]):
        is_checked_in (Union[Unset, bool]): Check-in status. True if the specified order item / everyone in a session
            (based on the request query), was checked in, false otherwise.
    """

    is_checke_in: Union[Unset, bool] = UNSET
    is_checked_in: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        is_checke_in = self.is_checke_in
        is_checked_in = self.is_checked_in

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if is_checke_in is not UNSET:
            field_dict["isCheckeIn"] = is_checke_in
        if is_checked_in is not UNSET:
            field_dict["isCheckedIn"] = is_checked_in

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        is_checke_in = d.pop("isCheckeIn", UNSET)

        is_checked_in = d.pop("isCheckedIn", UNSET)

        check_in = cls(
            is_checke_in=is_checke_in,
            is_checked_in=is_checked_in,
        )

        check_in.additional_properties = d
        return check_in

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class CreditCardCardType(str, Enum):
    VISA = "VISA"
    MASTERCARD = "MASTERCARD"
    AMEX = "AMEX"
    DINERS = "DINERS"
    DISCOVER = "DISCOVER"
    JCB = "JCB"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="Address")


@attr.s(auto_attribs=True)
class Address:
    """Address of a company, customer or product location.

    Attributes:
        address_line (Union[Unset, str]): Address line
        address_line_2 (Union[Unset, str]): Address line 2
        city (Union[Unset, str]): City name
        country_code (Union[Unset, str]): Country code
        latitude (Union[Unset, float]): Geolocation - latitude
        longitude (Union[Unset, float]): Geolocation - longitude
        post_code (Union[Unset, str]): Post Code
        state (Union[Unset, str]): State name
    """

    address_line: Union[Unset, str] = UNSET
    address_line_2: Union[Unset, str] = UNSET
    city: Union[Unset, str] = UNSET
    country_code: Union[Unset, str] = UNSET
    latitude: Union[Unset, float] = UNSET
    longitude: Union[Unset, float] = UNSET
    post_code: Union[Unset, str] = UNSET
    state: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        address_line = self.address_line
        address_line_2 = self.address_line_2
        city = self.city
        country_code = self.country_code
        latitude = self.latitude
        longitude = self.longitude
        post_code = self.post_code
        state = self.state

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if address_line is not UNSET:
            field_dict["addressLine"] = address_line
        if address_line_2 is not UNSET:
            field_dict["addressLine2"] = address_line_2
        if city is not UNSET:
            field_dict["city"] = city
        if country_code is not UNSET:
            field_dict["countryCode"] = country_code
        if latitude is not UNSET:
            field_dict["latitude"] = latitude
        if longitude is not UNSET:
            field_dict["longitude"] = longitude
        if post_code is not UNSET:
            field_dict["postCode"] = post_code
        if state is not UNSET:
            field_dict["state"] = state

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        address_line = d.pop("addressLine", UNSET)

        address_line_2 = d.pop("addressLine2", UNSET)

        city = d.pop("city", UNSET)

        country_code = d.pop("countryCode", UNSET)

        latitude = d.pop("latitude", UNSET)

        longitude = d.pop("longitude", UNSET)

        post_code = d.pop("postCode", UNSET)

        state = d.pop("state", UNSET)

        address = cls(
            address_line=address_line,
            address_line_2=address_line_2,
            city=city,
            country_code=country_code,
            latitude=latitude,
            longitude=longitude,
            post_code=post_code,
            state=state,
        )

        address.additional_properties = d
        return address

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.price_option import PriceOption
from ..types import UNSET, Unset

T = TypeVar("T", bound="SessionUpdateRequest")


@attr.s(auto_attribs=True)
class SessionUpdateRequest:
    """Updates session request data.

    Attributes:
        all_day (Union[Unset, bool]): If true, this session lasts all day and no time should be shown to customers.
            Technically the session will be from midnight to midnight.
        price_options (Union[Unset, List[PriceOption]]): List of price options, which will override the product level
            price. Price options have to be a subset of the product price options, thus you can not create new price
            options, use product update service to do so.
        seats (Union[Unset, int]): Update the total number of seats for this session. The total seats does not change
            after a booking is made. The total number of seats can not be less than 0.
        seats_available (Union[Unset, int]): Update the current availability for this session. The session total number
            of seats after updating the seats available can not be less than 0.
    """

    all_day: Union[Unset, bool] = UNSET
    price_options: Union[Unset, List[PriceOption]] = UNSET
    seats: Union[Unset, int] = UNSET
    seats_available: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        all_day = self.all_day
        price_options: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.price_options, Unset):
            price_options = []
            for price_options_item_data in self.price_options:
                price_options_item = price_options_item_data.to_dict()

                price_options.append(price_options_item)

        seats = self.seats
        seats_available = self.seats_available

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if all_day is not UNSET:
            field_dict["allDay"] = all_day
        if price_options is not UNSET:
            field_dict["priceOptions"] = price_options
        if seats is not UNSET:
            field_dict["seats"] = seats
        if seats_available is not UNSET:
            field_dict["seatsAvailable"] = seats_available

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        all_day = d.pop("allDay", UNSET)

        price_options = []
        _price_options = d.pop("priceOptions", UNSET)
        for price_options_item_data in _price_options or []:
            price_options_item = PriceOption.from_dict(price_options_item_data)

            price_options.append(price_options_item)

        seats = d.pop("seats", UNSET)

        seats_available = d.pop("seatsAvailable", UNSET)

        session_update_request = cls(
            all_day=all_day,
            price_options=price_options,
            seats=seats,
            seats_available=seats_available,
        )

        session_update_request.additional_properties = d
        return session_update_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.pickup_list import PickupList
from ..models.request_status import RequestStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="ResponsePickupList")


@attr.s(auto_attribs=True)
class ResponsePickupList:
    """
    Attributes:
        request_status (RequestStatus):
        pickup_list (Union[Unset, PickupList]): PickupList object. Contains a list of pickup locations.
    """

    request_status: RequestStatus
    pickup_list: Union[Unset, PickupList] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        request_status = self.request_status.to_dict()

        pickup_list: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.pickup_list, Unset):
            pickup_list = self.pickup_list.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "requestStatus": request_status,
            }
        )
        if pickup_list is not UNSET:
            field_dict["pickupList"] = pickup_list

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        request_status = RequestStatus.from_dict(d.pop("requestStatus"))

        _pickup_list = d.pop("pickupList", UNSET)
        pickup_list: Union[Unset, PickupList]
        if isinstance(_pickup_list, Unset):
            pickup_list = UNSET
        else:
            pickup_list = PickupList.from_dict(_pickup_list)

        response_pickup_list = cls(
            request_status=request_status,
            pickup_list=pickup_list,
        )

        response_pickup_list.additional_properties = d
        return response_pickup_list

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.customer import Customer
from ..models.request_status import RequestStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="ResponseCustomerList")


@attr.s(auto_attribs=True)
class ResponseCustomerList:
    """
    Attributes:
        request_status (RequestStatus):
        customers (Union[Unset, List[Customer]]):
    """

    request_status: RequestStatus
    customers: Union[Unset, List[Customer]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        request_status = self.request_status.to_dict()

        customers: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.customers, Unset):
            customers = []
            for customers_item_data in self.customers:
                customers_item = customers_item_data.to_dict()

                customers.append(customers_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "requestStatus": request_status,
            }
        )
        if customers is not UNSET:
            field_dict["customers"] = customers

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        request_status = RequestStatus.from_dict(d.pop("requestStatus"))

        customers = []
        _customers = d.pop("customers", UNSET)
        for customers_item_data in _customers or []:
            customers_item = Customer.from_dict(customers_item_data)

            customers.append(customers_item)

        response_customer_list = cls(
            request_status=request_status,
            customers=customers,
        )

        response_customer_list.additional_properties = d
        return response_customer_list

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict

import httpx

from ...client import Client
from ...types import Response


def _get_kwargs(
    product_code: str,
    media_id: int,
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/products/{productCode}/images/{mediaId}".format(
        client.base_url, productCode=product_code, mediaId=media_id
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "method": "delete",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _build_response(*, response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=None,
    )


def sync_detailed(
    product_code: str,
    media_id: int,
    *,
    client: Client,
) -> Response[Any]:
    """Remove product Image

     Removes product Image. Filename is mandatory. It has to be specified either in the attachment, part
    of the form-data parameter 'file', or in the attachment, as a form-data parameter 'filename', which
    is common format for PHP frameworks, which are sending an array of attachments.

    Args:
        product_code (str):
        media_id (int):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        product_code=product_code,
        media_id=media_id,
        client=client,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


async def asyncio_detailed(
    product_code: str,
    media_id: int,
    *,
    client: Client,
) -> Response[Any]:
    """Remove product Image

     Removes product Image. Filename is mandatory. It has to be specified either in the attachment, part
    of the form-data parameter 'file', or in the attachment, as a form-data parameter 'filename', which
    is common format for PHP frameworks, which are sending an array of attachments.

    Args:
        product_code (str):
        media_id (int):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        product_code=product_code,
        media_id=media_id,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.category import Category
from ..models.request_status import RequestStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="ResponseCategory")


@attr.s(auto_attribs=True)
class ResponseCategory:
    """
    Attributes:
        request_status (RequestStatus):
        category (Union[Unset, Category]): A Category is used to group products
    """

    request_status: RequestStatus
    category: Union[Unset, Category] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        request_status = self.request_status.to_dict()

        category: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.category, Unset):
            category = self.category.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "requestStatus": request_status,
            }
        )
        if category is not UNSET:
            field_dict["category"] = category

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        request_status = RequestStatus.from_dict(d.pop("requestStatus"))

        _category = d.pop("category", UNSET)
        category: Union[Unset, Category]
        if isinstance(_category, Unset):
            category = UNSET
        else:
            category = Category.from_dict(_category)

        response_category = cls(
            request_status=request_status,
            category=category,
        )

        response_category.additional_properties = d
        return response_category

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.price_option import PriceOption
from ..types import UNSET, Unset

T = TypeVar("T", bound="Session")


@attr.s(auto_attribs=True)
class Session:
    """A Session holds availability for a unique product / start time combination and also the rates for the session
    booking.

        Attributes:
            all_day (Union[Unset, bool]): If true, this session lasts all day and no time should be shown to customers.
                Technically the session will be from midnight to midnight.
            end_time (Union[Unset, datetime.datetime]): End time of this session. Used to show the customer how long that
                tour will last
            end_time_local (Union[Unset, str]): End time of this session in supplier's local timezone. Used to show the
                customer how long that tour will last
            id (Union[Unset, int]): Rezdy internal ID for this session
            price_options (Union[Unset, List[PriceOption]]): List of price options attached to this session. Most of the
                time they'll match the product's price options, but they can be used to change the price of specific dates/times
                (I.e. high/low season, weekday/weekend, etc.)
            product_code (Union[Unset, str]): Rezdy unique productCode linked to this session
            seats (Union[Unset, int]): Total number of seats for this session. Does not change after a booking is made
            seats_available (Union[Unset, int]): Current availability for this session.
            start_time (Union[Unset, datetime.datetime]): Start Time of this session. This time should be used when showing
                customers the booking date/time. It should be sent in BookingItem.startTime when making new bookings
            start_time_local (Union[Unset, str]): Start Time of this session in supplier's local timezone. This time should
                be used when showing customers the booking date/time. It should be sent in BookingItem.startTimeLocal when
                making new bookings
    """

    all_day: Union[Unset, bool] = UNSET
    end_time: Union[Unset, datetime.datetime] = UNSET
    end_time_local: Union[Unset, str] = UNSET
    id: Union[Unset, int] = UNSET
    price_options: Union[Unset, List[PriceOption]] = UNSET
    product_code: Union[Unset, str] = UNSET
    seats: Union[Unset, int] = UNSET
    seats_available: Union[Unset, int] = UNSET
    start_time: Union[Unset, datetime.datetime] = UNSET
    start_time_local: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        all_day = self.all_day
        end_time: Union[Unset, str] = UNSET
        if not isinstance(self.end_time, Unset):
            end_time = self.end_time.isoformat()

        end_time_local = self.end_time_local
        id = self.id
        price_options: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.price_options, Unset):
            price_options = []
            for price_options_item_data in self.price_options:
                price_options_item = price_options_item_data.to_dict()

                price_options.append(price_options_item)

        product_code = self.product_code
        seats = self.seats
        seats_available = self.seats_available
        start_time: Union[Unset, str] = UNSET
        if not isinstance(self.start_time, Unset):
            start_time = self.start_time.isoformat()

        start_time_local = self.start_time_local

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if all_day is not UNSET:
            field_dict["allDay"] = all_day
        if end_time is not UNSET:
            field_dict["endTime"] = end_time
        if end_time_local is not UNSET:
            field_dict["endTimeLocal"] = end_time_local
        if id is not UNSET:
            field_dict["id"] = id
        if price_options is not UNSET:
            field_dict["priceOptions"] = price_options
        if product_code is not UNSET:
            field_dict["productCode"] = product_code
        if seats is not UNSET:
            field_dict["seats"] = seats
        if seats_available is not UNSET:
            field_dict["seatsAvailable"] = seats_available
        if start_time is not UNSET:
            field_dict["startTime"] = start_time
        if start_time_local is not UNSET:
            field_dict["startTimeLocal"] = start_time_local

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        all_day = d.pop("allDay", UNSET)

        _end_time = d.pop("endTime", UNSET)
        end_time: Union[Unset, datetime.datetime]
        if isinstance(_end_time, Unset):
            end_time = UNSET
        else:
            end_time = isoparse(_end_time)

        end_time_local = d.pop("endTimeLocal", UNSET)

        id = d.pop("id", UNSET)

        price_options = []
        _price_options = d.pop("priceOptions", UNSET)
        for price_options_item_data in _price_options or []:
            price_options_item = PriceOption.from_dict(price_options_item_data)

            price_options.append(price_options_item)

        product_code = d.pop("productCode", UNSET)

        seats = d.pop("seats", UNSET)

        seats_available = d.pop("seatsAvailable", UNSET)

        _start_time = d.pop("startTime", UNSET)
        start_time: Union[Unset, datetime.datetime]
        if isinstance(_start_time, Unset):
            start_time = UNSET
        else:
            start_time = isoparse(_start_time)

        start_time_local = d.pop("startTimeLocal", UNSET)

        session = cls(
            all_day=all_day,
            end_time=end_time,
            end_time_local=end_time_local,
            id=id,
            price_options=price_options,
            product_code=product_code,
            seats=seats,
            seats_available=seats_available,
            start_time=start_time,
            start_time_local=start_time_local,
        )

        session.additional_properties = d
        return session

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.booking_field import BookingField
from ..models.booking_item_create import BookingItemCreate
from ..models.booking_update_status import BookingUpdateStatus
from ..models.customer import Customer
from ..models.user import User
from ..types import UNSET, Unset

T = TypeVar("T", bound="BookingUpdate")


@attr.s(auto_attribs=True)
class BookingUpdate:
    """Booking update object used to update a booking in Rezdy's system.

    Attributes:
        created_by (Union[Unset, User]): Internal Rezdy user details. This is a Rezdy application user who belongs to a
            Rezdy agent or supplier company.
        customer (Union[Unset, Customer]): The customer is the person making the booking, and most of the time paying
            for it.<br>It differs from Participants, who are the people attending a tour
        fields (Union[Unset, List[BookingField]]): List of custom fields that are required "once per booking" by all the
            products in this booking
        internal_notes (Union[Unset, str]): Comments only visible internally by the supplier
        items (Union[Unset, List[BookingItemCreate]]): List of items in this booking. A booking can contain multiple
            products. Each BookingItem is a separate product with its own set of quantities and participant details.
        order_number (Union[Unset, str]): Order number. This is the number you should give to customers and print on
            booking confirmations. Order number is generated by the system, therefore, even if it is specified in the
            booking request, it will be overwritten.
        reseller_comments (Union[Unset, str]): Comments only visible by the agent and the supplier. This should be used
            by the agent to send voucher numbers/redemption codes to suppliers.
        reseller_id (Union[Unset, int]): Rezdy internal ID of the agent company attached to this booking
        reseller_reference (Union[Unset, str]): External reseller reference, can be used to pass internal booking
            number. This reference will be shown to a supplier, also it will appear on reports and can be used to filter
            orders. Maxiumum number of characters is 30
        reseller_user (Union[Unset, User]): Internal Rezdy user details. This is a Rezdy application user who belongs to
            a Rezdy agent or supplier company.
        status (Union[Unset, BookingUpdateStatus]): Status of this booking
    """

    created_by: Union[Unset, User] = UNSET
    customer: Union[Unset, Customer] = UNSET
    fields: Union[Unset, List[BookingField]] = UNSET
    internal_notes: Union[Unset, str] = UNSET
    items: Union[Unset, List[BookingItemCreate]] = UNSET
    order_number: Union[Unset, str] = UNSET
    reseller_comments: Union[Unset, str] = UNSET
    reseller_id: Union[Unset, int] = UNSET
    reseller_reference: Union[Unset, str] = UNSET
    reseller_user: Union[Unset, User] = UNSET
    status: Union[Unset, BookingUpdateStatus] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        created_by: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.created_by, Unset):
            created_by = self.created_by.to_dict()

        customer: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.customer, Unset):
            customer = self.customer.to_dict()

        fields: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.fields, Unset):
            fields = []
            for fields_item_data in self.fields:
                fields_item = fields_item_data.to_dict()

                fields.append(fields_item)

        internal_notes = self.internal_notes
        items: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.items, Unset):
            items = []
            for items_item_data in self.items:
                items_item = items_item_data.to_dict()

                items.append(items_item)

        order_number = self.order_number
        reseller_comments = self.reseller_comments
        reseller_id = self.reseller_id
        reseller_reference = self.reseller_reference
        reseller_user: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.reseller_user, Unset):
            reseller_user = self.reseller_user.to_dict()

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if created_by is not UNSET:
            field_dict["createdBy"] = created_by
        if customer is not UNSET:
            field_dict["customer"] = customer
        if fields is not UNSET:
            field_dict["fields"] = fields
        if internal_notes is not UNSET:
            field_dict["internalNotes"] = internal_notes
        if items is not UNSET:
            field_dict["items"] = items
        if order_number is not UNSET:
            field_dict["orderNumber"] = order_number
        if reseller_comments is not UNSET:
            field_dict["resellerComments"] = reseller_comments
        if reseller_id is not UNSET:
            field_dict["resellerId"] = reseller_id
        if reseller_reference is not UNSET:
            field_dict["resellerReference"] = reseller_reference
        if reseller_user is not UNSET:
            field_dict["resellerUser"] = reseller_user
        if status is not UNSET:
            field_dict["status"] = status

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _created_by = d.pop("createdBy", UNSET)
        created_by: Union[Unset, User]
        if isinstance(_created_by, Unset):
            created_by = UNSET
        else:
            created_by = User.from_dict(_created_by)

        _customer = d.pop("customer", UNSET)
        customer: Union[Unset, Customer]
        if isinstance(_customer, Unset):
            customer = UNSET
        else:
            customer = Customer.from_dict(_customer)

        fields = []
        _fields = d.pop("fields", UNSET)
        for fields_item_data in _fields or []:
            fields_item = BookingField.from_dict(fields_item_data)

            fields.append(fields_item)

        internal_notes = d.pop("internalNotes", UNSET)

        items = []
        _items = d.pop("items", UNSET)
        for items_item_data in _items or []:
            items_item = BookingItemCreate.from_dict(items_item_data)

            items.append(items_item)

        order_number = d.pop("orderNumber", UNSET)

        reseller_comments = d.pop("resellerComments", UNSET)

        reseller_id = d.pop("resellerId", UNSET)

        reseller_reference = d.pop("resellerReference", UNSET)

        _reseller_user = d.pop("resellerUser", UNSET)
        reseller_user: Union[Unset, User]
        if isinstance(_reseller_user, Unset):
            reseller_user = UNSET
        else:
            reseller_user = User.from_dict(_reseller_user)

        _status = d.pop("status", UNSET)
        status: Union[Unset, BookingUpdateStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = BookingUpdateStatus(_status)

        booking_update = cls(
            created_by=created_by,
            customer=customer,
            fields=fields,
            internal_notes=internal_notes,
            items=items,
            order_number=order_number,
            reseller_comments=reseller_comments,
            reseller_id=reseller_id,
            reseller_reference=reseller_reference,
            reseller_user=reseller_user,
            status=status,
        )

        booking_update.additional_properties = d
        return booking_update

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ResourceType(str, Enum):
    RESOURCE = "RESOURCE"
    BUS = "BUS"
    COACH = "COACH"
    COASTER = "COASTER"
    VAN = "VAN"
    CAR = "CAR"
    MOTORCYCLE = "MOTORCYCLE"
    BOAT = "BOAT"
    VEHICLE = "VEHICLE"
    KAYAK = "KAYAK"
    GUIDE = "GUIDE"
    DRIVER = "DRIVER"
    PILOT = "PILOT"
    INSTRUCTOR = "INSTRUCTOR"
    ROOM = "ROOM"
    VENUE = "VENUE"
    HORSE = "HORSE"
    HELICOPTER = "HELICOPTER"
    PLANE = "PLANE"
    RAFT = "RAFT"
    BALLOON = "BALLOON"
    TRACK = "TRACK"
    FIELD = "FIELD"
    RING = "RING"
    CAMEL = "CAMEL"
    BIKE = "BIKE"
    SEGWAY = "SEGWAY"
    SKI = "SKI"
    SNOWBOARD = "SNOWBOARD"
    PADDLEBOARD = "PADDLEBOARD"
    EQUIPMENT = "EQUIPMENT"
    SKIPPER = "SKIPPER"
    CREW = "CREW"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Union

import httpx

from ...client import Client
from ...types import UNSET, Response, Unset


def _get_kwargs(
    resource_id: int,
    *,
    client: Client,
    start_time: Union[Unset, None, str] = UNSET,
    end_time: Union[Unset, None, str] = UNSET,
    start_time_local: Union[Unset, None, str] = UNSET,
    end_time_local: Union[Unset, None, str] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    offset: Union[Unset, None, int] = 0,
) -> Dict[str, Any]:
    url = "{}/resources/{resourceId}/sessions".format(client.base_url, resourceId=resource_id)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["startTime"] = start_time

    params["endTime"] = end_time

    params["startTimeLocal"] = start_time_local

    params["endTimeLocal"] = end_time_local

    params["limit"] = limit

    params["offset"] = offset

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _build_response(*, response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=None,
    )


def sync_detailed(
    resource_id: int,
    *,
    client: Client,
    start_time: Union[Unset, None, str] = UNSET,
    end_time: Union[Unset, None, str] = UNSET,
    start_time_local: Union[Unset, None, str] = UNSET,
    end_time_local: Union[Unset, None, str] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    offset: Union[Unset, None, int] = 0,
) -> Response[Any]:
    """Get resource sessions

     Retrieves all sessions for the specified resource within the start/end datetime range. Pagination
    using limit and offset is applied to the result list.

    Args:
        resource_id (int):
        start_time (Union[Unset, None, str]):
        end_time (Union[Unset, None, str]):
        start_time_local (Union[Unset, None, str]):
        end_time_local (Union[Unset, None, str]):
        limit (Union[Unset, None, int]):
        offset (Union[Unset, None, int]):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        resource_id=resource_id,
        client=client,
        start_time=start_time,
        end_time=end_time,
        start_time_local=start_time_local,
        end_time_local=end_time_local,
        limit=limit,
        offset=offset,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


async def asyncio_detailed(
    resource_id: int,
    *,
    client: Client,
    start_time: Union[Unset, None, str] = UNSET,
    end_time: Union[Unset, None, str] = UNSET,
    start_time_local: Union[Unset, None, str] = UNSET,
    end_time_local: Union[Unset, None, str] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    offset: Union[Unset, None, int] = 0,
) -> Response[Any]:
    """Get resource sessions

     Retrieves all sessions for the specified resource within the start/end datetime range. Pagination
    using limit and offset is applied to the result list.

    Args:
        resource_id (int):
        start_time (Union[Unset, None, str]):
        end_time (Union[Unset, None, str]):
        start_time_local (Union[Unset, None, str]):
        end_time_local (Union[Unset, None, str]):
        limit (Union[Unset, None, int]):
        offset (Union[Unset, None, int]):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        resource_id=resource_id,
        client=client,
        start_time=start_time,
        end_time=end_time,
        start_time_local=start_time_local,
        end_time_local=end_time_local,
        limit=limit,
        offset=offset,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr
from dateutil.parser import isoparse

from ..models.address import Address
from ..models.booking_field import BookingField
from ..models.extra import Extra
from ..models.image import Image
from ..models.price_option import PriceOption
from ..models.product_agent_payment_type import ProductAgentPaymentType
from ..models.product_barcode_output_type import ProductBarcodeOutputType
from ..models.product_booking_mode import ProductBookingMode
from ..models.product_confirm_mode import ProductConfirmMode
from ..models.product_currency import ProductCurrency
from ..models.product_product_type import ProductProductType
from ..models.product_qr_code_type import ProductQrCodeType
from ..models.product_seo_tag import ProductSeoTag
from ..models.tax import Tax
from ..models.video import Video
from ..types import UNSET, Unset

T = TypeVar("T", bound="Product")


@attr.s(auto_attribs=True)
class Product:
    """Product object. Holds general details and settings of a specific tour, activity or event.

    Attributes:
        booking_fields (List[BookingField]): List of booking fields required for this product
        booking_mode (ProductBookingMode): Booking mode. Determines if this product needs availability or can be booked
            for any date.
        confirm_mode (ProductConfirmMode): Confirmation mode. Determines if bookings are automatically confirmed or it
            they are pending
        description (str): Long product description, is between 100 and 15000 characters
        price_options (List[PriceOption]): List of price options belonging to this product.
        product_code (str): Rezdy-generated unique Product code. Used by agents and for API calls
        quantity_required (bool): Does this product require a quantity to be booked? True for most products. Can be
            false if the supplier can only provide one quantity at any single time. (I.e. private charters)
        short_description (str): Product description is between 15 and 240 characters
        supplier_id (int): Rezdy internal ID of the company supplying this product
        supplier_name (str): Name of the company supplying this product
        timezone (str): Timezone used by this product and supplier. All Times must be converted to this timezone before
            being displayed to customers
        unit_label (str): What a quantity for this product is. It can be people (I.e. participant, passenger, diver) or
            objects (Kayak, Helicopter, etc.)
        unit_label_plural (str): Plural version of unitLabel
        additional_information (Union[Unset, str]): Additional information for the product, that should be sent after a
            booking is completed (i.e. by email) to the customer. Useful for integration, when manual control of the entire
            customer booking experience is wanted, and the automatic confirmation e-mail from Rezdy had been suppressed.
        advertised_price (Union[Unset, float]): General price indication for this product. It represents a display price
            only, therefore it does not affect a real booking price, which is calculated based on the price options.
        agent_payment_type (Union[Unset, ProductAgentPaymentType]): If you are an agent, payment rules setup by the
            supplier for you to book this product
        api_booking_supported (Union[Unset, bool]):
        barcode_output_type (Union[Unset, ProductBarcodeOutputType]): Specifies how to output the barcodes when this
            product is booked. Valid types are:<br><li>PARTICIPANT: Barcodes will be generated by rezdy for each participant
            when an booking is created for this product</li><li>ORDER: Barcodes will be generated by rezdy per booking</li>
        cancellation_policy_days (Union[Unset, int]):  Supplier's Cancellation policy. Number of days before the tour a
            cancellation is allowed with full refund.<br>This is only used for automated payments (PAYOUTS) bookings
        charter (Union[Unset, bool]): A charter product means each session can only have a single booking, whatever the
            number of seats booked.
        commission_includes_extras (Union[Unset, bool]): True if agent receive commission from extras, false otherwise.
        confirm_mode_min_participants (Union[Unset, int]): If confirmMode is MANUAL_THEN_AUTO or AUTO_THEN_MANUAL,
            determines the minimum number of participants per booking to trigger the change
        currency (Union[Unset, ProductCurrency]): Product prices Currency
        date_created (Union[Unset, datetime.datetime]): The product creation date
        date_updated (Union[Unset, datetime.datetime]): * The date of the last product update
        duration_minutes (Union[Unset, int]): * Duration of the product in minutes.
        extras (Union[Unset, List[Extra]]): List of extras that can be booked with this product
        general_terms (Union[Unset, str]): General terms and conditions for all products from this supplier
        images (Union[Unset, List[Image]]): List of images showcasing this product
        internal_code (Union[Unset, str]): Supplier-defined product code, used internally by the ther supplier
        languages (Union[Unset, List[str]]): List of product languages. The format of the language is ISO 639 two-letter
            code with BCP 47 language variants, separated by underscore e.g. en_au.
        location_address (Union[Unset, Address]): Address of a company, customer or product location.
        max_commission_net_rate (Union[Unset, float]): Maximum commission amount you can receive as an agent, when the
            supplier setup a net rate (Automated payments Rezdy fee is not included in the amount)
        max_commission_percent (Union[Unset, float]): Maximum commission % you can receive as an agent, when the
            supplier setup a percentage (Automated payments Rezdy fee is not included in the amount)
        minimum_notice_minutes (Union[Unset, int]): * Minimum book ahead internal before session start time in minutes.
        multi_product_booking_supported (Union[Unset, bool]):
        name (Union[Unset, str]): Product name
        pickup_id (Union[Unset, int]): * If pickups are configured for this product, the field will contain the id of
            the pickup location list created by the supplier.
        product_seo_tags (Union[Unset, List[ProductSeoTag]]): This will store product meta data such as title and
            description
        product_type (Union[Unset, ProductProductType]): Type of this product
        qr_code_type (Union[Unset, ProductQrCodeType]): Specifies the method how QR Codes will be generated for this
            product. Valid types are:<br><li>INTERNAL: QR Codes will be generated by rezdy for each participant when an
            order is created for this product</li><li>EXTERNAL: QR Codes will be randomly taken from a list of imported QR
            Codes</li><p>If nothing is specified, then no QR Codes will be generated when an order is created for this
            product
        quantity_required_max (Union[Unset, int]): Represent the max booking quantity for the product. It can be setup
            for a supplier product. For a successful booking of the product, the total number of participants (regardless of
            pricing options), per booking item in the booking request, have to be lesser or equal than this value.
        quantity_required_min (Union[Unset, int]): Represent the min booking quantity for the product. It can be setup
            for a supplier product. For a successful booking of the product, the total number of participants (regardless of
            pricing options), per booking item in the booking request, have to be greater or equal than this value.
        supplier_alias (Union[Unset, str]): Alias of the company supplying this product. Company alias is a unique key
            and should be used to retrieve company details or in filters
        tags (Union[Unset, List[str]]): List of tags related to the product. The format is [TAG_TYPE]:[TAG_VALUE] e.g.
            TYPE:ACTIVITY, CATEGORY:ABSEILING, INTENSITY:RELAXED, ACCESSIBILITY:VISION_IMPAIRED
        taxes (Union[Unset, List[Tax]]): List of taxes/fees associated with the product
        terms (Union[Unset, str]): Specific terms and conditions for this product
        videos (Union[Unset, List[Video]]): <p>List of videos showcasing this product <br/>Videos will only be returned
            when a single product is loaded.</p>
        wait_listing_enabled (Union[Unset, bool]): Signifies that customers will still be able to book this product even
            when there is not enough availability. Orders will have "On Hold" status, and no payment will be processed
        xero_account (Union[Unset, str]): Supplier Xero account for this product
    """

    booking_fields: List[BookingField]
    booking_mode: ProductBookingMode
    confirm_mode: ProductConfirmMode
    description: str
    price_options: List[PriceOption]
    product_code: str
    quantity_required: bool
    short_description: str
    supplier_id: int
    supplier_name: str
    timezone: str
    unit_label: str
    unit_label_plural: str
    additional_information: Union[Unset, str] = UNSET
    advertised_price: Union[Unset, float] = UNSET
    agent_payment_type: Union[Unset, ProductAgentPaymentType] = UNSET
    api_booking_supported: Union[Unset, bool] = UNSET
    barcode_output_type: Union[Unset, ProductBarcodeOutputType] = UNSET
    cancellation_policy_days: Union[Unset, int] = UNSET
    charter: Union[Unset, bool] = UNSET
    commission_includes_extras: Union[Unset, bool] = UNSET
    confirm_mode_min_participants: Union[Unset, int] = UNSET
    currency: Union[Unset, ProductCurrency] = UNSET
    date_created: Union[Unset, datetime.datetime] = UNSET
    date_updated: Union[Unset, datetime.datetime] = UNSET
    duration_minutes: Union[Unset, int] = UNSET
    extras: Union[Unset, List[Extra]] = UNSET
    general_terms: Union[Unset, str] = UNSET
    images: Union[Unset, List[Image]] = UNSET
    internal_code: Union[Unset, str] = UNSET
    languages: Union[Unset, List[str]] = UNSET
    location_address: Union[Unset, Address] = UNSET
    max_commission_net_rate: Union[Unset, float] = UNSET
    max_commission_percent: Union[Unset, float] = UNSET
    minimum_notice_minutes: Union[Unset, int] = UNSET
    multi_product_booking_supported: Union[Unset, bool] = UNSET
    name: Union[Unset, str] = UNSET
    pickup_id: Union[Unset, int] = UNSET
    product_seo_tags: Union[Unset, List[ProductSeoTag]] = UNSET
    product_type: Union[Unset, ProductProductType] = UNSET
    qr_code_type: Union[Unset, ProductQrCodeType] = UNSET
    quantity_required_max: Union[Unset, int] = UNSET
    quantity_required_min: Union[Unset, int] = UNSET
    supplier_alias: Union[Unset, str] = UNSET
    tags: Union[Unset, List[str]] = UNSET
    taxes: Union[Unset, List[Tax]] = UNSET
    terms: Union[Unset, str] = UNSET
    videos: Union[Unset, List[Video]] = UNSET
    wait_listing_enabled: Union[Unset, bool] = UNSET
    xero_account: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        booking_fields = []
        for booking_fields_item_data in self.booking_fields:
            booking_fields_item = booking_fields_item_data.to_dict()

            booking_fields.append(booking_fields_item)

        booking_mode = self.booking_mode.value

        confirm_mode = self.confirm_mode.value

        description = self.description
        price_options = []
        for price_options_item_data in self.price_options:
            price_options_item = price_options_item_data.to_dict()

            price_options.append(price_options_item)

        product_code = self.product_code
        quantity_required = self.quantity_required
        short_description = self.short_description
        supplier_id = self.supplier_id
        supplier_name = self.supplier_name
        timezone = self.timezone
        unit_label = self.unit_label
        unit_label_plural = self.unit_label_plural
        additional_information = self.additional_information
        advertised_price = self.advertised_price
        agent_payment_type: Union[Unset, str] = UNSET
        if not isinstance(self.agent_payment_type, Unset):
            agent_payment_type = self.agent_payment_type.value

        api_booking_supported = self.api_booking_supported
        barcode_output_type: Union[Unset, str] = UNSET
        if not isinstance(self.barcode_output_type, Unset):
            barcode_output_type = self.barcode_output_type.value

        cancellation_policy_days = self.cancellation_policy_days
        charter = self.charter
        commission_includes_extras = self.commission_includes_extras
        confirm_mode_min_participants = self.confirm_mode_min_participants
        currency: Union[Unset, str] = UNSET
        if not isinstance(self.currency, Unset):
            currency = self.currency.value

        date_created: Union[Unset, str] = UNSET
        if not isinstance(self.date_created, Unset):
            date_created = self.date_created.isoformat()

        date_updated: Union[Unset, str] = UNSET
        if not isinstance(self.date_updated, Unset):
            date_updated = self.date_updated.isoformat()

        duration_minutes = self.duration_minutes
        extras: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.extras, Unset):
            extras = []
            for extras_item_data in self.extras:
                extras_item = extras_item_data.to_dict()

                extras.append(extras_item)

        general_terms = self.general_terms
        images: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.images, Unset):
            images = []
            for images_item_data in self.images:
                images_item = images_item_data.to_dict()

                images.append(images_item)

        internal_code = self.internal_code
        languages: Union[Unset, List[str]] = UNSET
        if not isinstance(self.languages, Unset):
            languages = self.languages

        location_address: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.location_address, Unset):
            location_address = self.location_address.to_dict()

        max_commission_net_rate = self.max_commission_net_rate
        max_commission_percent = self.max_commission_percent
        minimum_notice_minutes = self.minimum_notice_minutes
        multi_product_booking_supported = self.multi_product_booking_supported
        name = self.name
        pickup_id = self.pickup_id
        product_seo_tags: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.product_seo_tags, Unset):
            product_seo_tags = []
            for product_seo_tags_item_data in self.product_seo_tags:
                product_seo_tags_item = product_seo_tags_item_data.to_dict()

                product_seo_tags.append(product_seo_tags_item)

        product_type: Union[Unset, str] = UNSET
        if not isinstance(self.product_type, Unset):
            product_type = self.product_type.value

        qr_code_type: Union[Unset, str] = UNSET
        if not isinstance(self.qr_code_type, Unset):
            qr_code_type = self.qr_code_type.value

        quantity_required_max = self.quantity_required_max
        quantity_required_min = self.quantity_required_min
        supplier_alias = self.supplier_alias
        tags: Union[Unset, List[str]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags

        taxes: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.taxes, Unset):
            taxes = []
            for taxes_item_data in self.taxes:
                taxes_item = taxes_item_data.to_dict()

                taxes.append(taxes_item)

        terms = self.terms
        videos: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.videos, Unset):
            videos = []
            for videos_item_data in self.videos:
                videos_item = videos_item_data.to_dict()

                videos.append(videos_item)

        wait_listing_enabled = self.wait_listing_enabled
        xero_account = self.xero_account

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "bookingFields": booking_fields,
                "bookingMode": booking_mode,
                "confirmMode": confirm_mode,
                "description": description,
                "priceOptions": price_options,
                "productCode": product_code,
                "quantityRequired": quantity_required,
                "shortDescription": short_description,
                "supplierId": supplier_id,
                "supplierName": supplier_name,
                "timezone": timezone,
                "unitLabel": unit_label,
                "unitLabelPlural": unit_label_plural,
            }
        )
        if additional_information is not UNSET:
            field_dict["additionalInformation"] = additional_information
        if advertised_price is not UNSET:
            field_dict["advertisedPrice"] = advertised_price
        if agent_payment_type is not UNSET:
            field_dict["agentPaymentType"] = agent_payment_type
        if api_booking_supported is not UNSET:
            field_dict["apiBookingSupported"] = api_booking_supported
        if barcode_output_type is not UNSET:
            field_dict["barcodeOutputType"] = barcode_output_type
        if cancellation_policy_days is not UNSET:
            field_dict["cancellationPolicyDays"] = cancellation_policy_days
        if charter is not UNSET:
            field_dict["charter"] = charter
        if commission_includes_extras is not UNSET:
            field_dict["commissionIncludesExtras"] = commission_includes_extras
        if confirm_mode_min_participants is not UNSET:
            field_dict["confirmModeMinParticipants"] = confirm_mode_min_participants
        if currency is not UNSET:
            field_dict["currency"] = currency
        if date_created is not UNSET:
            field_dict["dateCreated"] = date_created
        if date_updated is not UNSET:
            field_dict["dateUpdated"] = date_updated
        if duration_minutes is not UNSET:
            field_dict["durationMinutes"] = duration_minutes
        if extras is not UNSET:
            field_dict["extras"] = extras
        if general_terms is not UNSET:
            field_dict["generalTerms"] = general_terms
        if images is not UNSET:
            field_dict["images"] = images
        if internal_code is not UNSET:
            field_dict["internalCode"] = internal_code
        if languages is not UNSET:
            field_dict["languages"] = languages
        if location_address is not UNSET:
            field_dict["locationAddress"] = location_address
        if max_commission_net_rate is not UNSET:
            field_dict["maxCommissionNetRate"] = max_commission_net_rate
        if max_commission_percent is not UNSET:
            field_dict["maxCommissionPercent"] = max_commission_percent
        if minimum_notice_minutes is not UNSET:
            field_dict["minimumNoticeMinutes"] = minimum_notice_minutes
        if multi_product_booking_supported is not UNSET:
            field_dict["multiProductBookingSupported"] = multi_product_booking_supported
        if name is not UNSET:
            field_dict["name"] = name
        if pickup_id is not UNSET:
            field_dict["pickupId"] = pickup_id
        if product_seo_tags is not UNSET:
            field_dict["productSeoTags"] = product_seo_tags
        if product_type is not UNSET:
            field_dict["productType"] = product_type
        if qr_code_type is not UNSET:
            field_dict["qrCodeType"] = qr_code_type
        if quantity_required_max is not UNSET:
            field_dict["quantityRequiredMax"] = quantity_required_max
        if quantity_required_min is not UNSET:
            field_dict["quantityRequiredMin"] = quantity_required_min
        if supplier_alias is not UNSET:
            field_dict["supplierAlias"] = supplier_alias
        if tags is not UNSET:
            field_dict["tags"] = tags
        if taxes is not UNSET:
            field_dict["taxes"] = taxes
        if terms is not UNSET:
            field_dict["terms"] = terms
        if videos is not UNSET:
            field_dict["videos"] = videos
        if wait_listing_enabled is not UNSET:
            field_dict["waitListingEnabled"] = wait_listing_enabled
        if xero_account is not UNSET:
            field_dict["xeroAccount"] = xero_account

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        booking_fields = []
        _booking_fields = d.pop("bookingFields")
        for booking_fields_item_data in _booking_fields:
            booking_fields_item = BookingField.from_dict(booking_fields_item_data)

            booking_fields.append(booking_fields_item)

        booking_mode = ProductBookingMode(d.pop("bookingMode"))

        confirm_mode = ProductConfirmMode(d.pop("confirmMode"))

        description = d.pop("description")

        price_options = []
        _price_options = d.pop("priceOptions")
        for price_options_item_data in _price_options:
            price_options_item = PriceOption.from_dict(price_options_item_data)

            price_options.append(price_options_item)

        product_code = d.pop("productCode")

        quantity_required = d.pop("quantityRequired")

        short_description = d.pop("shortDescription")

        supplier_id = d.pop("supplierId")

        supplier_name = d.pop("supplierName")

        timezone = d.pop("timezone")

        unit_label = d.pop("unitLabel")

        unit_label_plural = d.pop("unitLabelPlural")

        additional_information = d.pop("additionalInformation", UNSET)

        advertised_price = d.pop("advertisedPrice", UNSET)

        _agent_payment_type = d.pop("agentPaymentType", UNSET)
        agent_payment_type: Union[Unset, ProductAgentPaymentType]
        if isinstance(_agent_payment_type, Unset):
            agent_payment_type = UNSET
        else:
            agent_payment_type = ProductAgentPaymentType(_agent_payment_type)

        api_booking_supported = d.pop("apiBookingSupported", UNSET)

        _barcode_output_type = d.pop("barcodeOutputType", UNSET)
        barcode_output_type: Union[Unset, ProductBarcodeOutputType]
        if isinstance(_barcode_output_type, Unset):
            barcode_output_type = UNSET
        else:
            barcode_output_type = ProductBarcodeOutputType(_barcode_output_type)

        cancellation_policy_days = d.pop("cancellationPolicyDays", UNSET)

        charter = d.pop("charter", UNSET)

        commission_includes_extras = d.pop("commissionIncludesExtras", UNSET)

        confirm_mode_min_participants = d.pop("confirmModeMinParticipants", UNSET)

        _currency = d.pop("currency", UNSET)
        currency: Union[Unset, ProductCurrency]
        if isinstance(_currency, Unset):
            currency = UNSET
        else:
            currency = ProductCurrency(_currency)

        _date_created = d.pop("dateCreated", UNSET)
        date_created: Union[Unset, datetime.datetime]
        if isinstance(_date_created, Unset):
            date_created = UNSET
        else:
            date_created = isoparse(_date_created)

        _date_updated = d.pop("dateUpdated", UNSET)
        date_updated: Union[Unset, datetime.datetime]
        if isinstance(_date_updated, Unset):
            date_updated = UNSET
        else:
            date_updated = isoparse(_date_updated)

        duration_minutes = d.pop("durationMinutes", UNSET)

        extras = []
        _extras = d.pop("extras", UNSET)
        for extras_item_data in _extras or []:
            extras_item = Extra.from_dict(extras_item_data)

            extras.append(extras_item)

        general_terms = d.pop("generalTerms", UNSET)

        images = []
        _images = d.pop("images", UNSET)
        for images_item_data in _images or []:
            images_item = Image.from_dict(images_item_data)

            images.append(images_item)

        internal_code = d.pop("internalCode", UNSET)

        languages = cast(List[str], d.pop("languages", UNSET))

        _location_address = d.pop("locationAddress", UNSET)
        location_address: Union[Unset, Address]
        if isinstance(_location_address, Unset):
            location_address = UNSET
        else:
            location_address = Address.from_dict(_location_address)

        max_commission_net_rate = d.pop("maxCommissionNetRate", UNSET)

        max_commission_percent = d.pop("maxCommissionPercent", UNSET)

        minimum_notice_minutes = d.pop("minimumNoticeMinutes", UNSET)

        multi_product_booking_supported = d.pop("multiProductBookingSupported", UNSET)

        name = d.pop("name", UNSET)

        pickup_id = d.pop("pickupId", UNSET)

        product_seo_tags = []
        _product_seo_tags = d.pop("productSeoTags", UNSET)
        for product_seo_tags_item_data in _product_seo_tags or []:
            product_seo_tags_item = ProductSeoTag.from_dict(product_seo_tags_item_data)

            product_seo_tags.append(product_seo_tags_item)

        _product_type = d.pop("productType", UNSET)
        product_type: Union[Unset, ProductProductType]
        if isinstance(_product_type, Unset):
            product_type = UNSET
        else:
            product_type = ProductProductType(_product_type)

        _qr_code_type = d.pop("qrCodeType", UNSET)
        qr_code_type: Union[Unset, ProductQrCodeType]
        if isinstance(_qr_code_type, Unset):
            qr_code_type = UNSET
        else:
            qr_code_type = ProductQrCodeType(_qr_code_type)

        quantity_required_max = d.pop("quantityRequiredMax", UNSET)

        quantity_required_min = d.pop("quantityRequiredMin", UNSET)

        supplier_alias = d.pop("supplierAlias", UNSET)

        tags = cast(List[str], d.pop("tags", UNSET))

        taxes = []
        _taxes = d.pop("taxes", UNSET)
        for taxes_item_data in _taxes or []:
            taxes_item = Tax.from_dict(taxes_item_data)

            taxes.append(taxes_item)

        terms = d.pop("terms", UNSET)

        videos = []
        _videos = d.pop("videos", UNSET)
        for videos_item_data in _videos or []:
            videos_item = Video.from_dict(videos_item_data)

            videos.append(videos_item)

        wait_listing_enabled = d.pop("waitListingEnabled", UNSET)

        xero_account = d.pop("xeroAccount", UNSET)

        product = cls(
            booking_fields=booking_fields,
            booking_mode=booking_mode,
            confirm_mode=confirm_mode,
            description=description,
            price_options=price_options,
            product_code=product_code,
            quantity_required=quantity_required,
            short_description=short_description,
            supplier_id=supplier_id,
            supplier_name=supplier_name,
            timezone=timezone,
            unit_label=unit_label,
            unit_label_plural=unit_label_plural,
            additional_information=additional_information,
            advertised_price=advertised_price,
            agent_payment_type=agent_payment_type,
            api_booking_supported=api_booking_supported,
            barcode_output_type=barcode_output_type,
            cancellation_policy_days=cancellation_policy_days,
            charter=charter,
            commission_includes_extras=commission_includes_extras,
            confirm_mode_min_participants=confirm_mode_min_participants,
            currency=currency,
            date_created=date_created,
            date_updated=date_updated,
            duration_minutes=duration_minutes,
            extras=extras,
            general_terms=general_terms,
            images=images,
            internal_code=internal_code,
            languages=languages,
            location_address=location_address,
            max_commission_net_rate=max_commission_net_rate,
            max_commission_percent=max_commission_percent,
            minimum_notice_minutes=minimum_notice_minutes,
            multi_product_booking_supported=multi_product_booking_supported,
            name=name,
            pickup_id=pickup_id,
            product_seo_tags=product_seo_tags,
            product_type=product_type,
            qr_code_type=qr_code_type,
            quantity_required_max=quantity_required_max,
            quantity_required_min=quantity_required_min,
            supplier_alias=supplier_alias,
            tags=tags,
            taxes=taxes,
            terms=terms,
            videos=videos,
            wait_listing_enabled=wait_listing_enabled,
            xero_account=xero_account,
        )

        product.additional_properties = d
        return product

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
